//Core Global Variables
$Adventure::MovementSpeed = $Adventure::MovementSpeed; //Movement Speed for main datablock (and its children) - it is set in "Datablocks_Player.cs"

//Climbing Global Variables
$Adventure::LedgeHangLen = 0.7; //Climbing: Ideal ledge length
$Adventure::HangHeight = 1.2; //1.4; //Climbing: Minimum Height for Hang Box Check
$Adventure::PosA_offset = 0.13; //Vertical offset for calculating ledge search starting point
$Adventure::PosB_offset = 0.23; //Vertical offset for calculating ledge gap search starting point (the space above a ledge)
$Adventure::SmartDropZ = -0.97; //minimum look vector z component for Smart Drop to activate

//Debugging Mode Variables
$Adventure::HandTime = 1; //Fancy-Hands: Hand update timeout
$Adventure::HiBrickTime = 500; //Debugging: Ledge Highlighting Brick lifetime
$Adventure::RayDelTime = 50; //Debugging: Raycast Shape lifetime
$Adventure::BoxDelTime = 2000; //Debugging: Raycast Shape lifetime


/////////
//Start//
/////////

//Adventure Group Support
if(!isObject($AdventureList))
{
	$AdventureList = new SimGroup()
	{
		class = "AdventureList";
	};
}

//Adventure Global Silent Loop
function AdventureGlobalSilentLoop(%player)
{
	if(isObject(%player)) //if player arg. defined
		$AdventureList.add(%player); //add player to Adventure list
	
	for(%a=0;%a<$AdventureList.getCount();%a++) //Search through Adventure players
	{
		%player = $AdventureList.getObject(%a); //get player from Adventure list
		if(!isObject(%player) || %player.getState() $= "Dead" || !(%data = %player.getDatablock()).canAdventure) //if player DNE, or player is dead, or datablock is not an Adventurer
		{
			$AdventureList.remove(%player); //remove player from Adventure list
			continue; //skip player, and continue looping
		}
	//	AdventureAimCheck(%player,%data); //Crosshair integration
	//	AdventureSprintCheck(%player,%data); //Crosshair integration
		AdventureClimbCheck(%player,%data); //Climbing integration
		%isAdventurer = true; //since at least one player was Adventure-ready
	}

	if(%isAdventurer) //if an Adventurer exists
	{
		cancel($Adventure::SilentLoop); //cancel any pending Silent Loop schedules
		$Adventure::SilentLoop = scheduleNoQuota(1,0,AdventureGlobalSilentLoop); //continue the loop
	}
}


/////////////
//Sprinting//
/////////////

//Sprint Toggle Check
function AdventureSprintCheck(%player,%data)
{
	%eyeVec = %player.getEyeVector();
	%z = getWord(%eyeVec,2);
	%runReady = %z < -0.8;
	
	if(%runReady && %data.getName() $= "PlayerAdventure") //if 'isAdvRunReady' bool is true (tapping Jump button)
		%player.changedatablock(PlayerAdventureRun); //change to Run data
	else if(!%runReady && %data.canAdventureRun) //if in Run data
		%player.changedatablock(PlayerAdventure); //switch to main datablock
}

package AdventurePlayer
{
	//on Datablock Change
	function Armor::onNewDatablock(%data,%player)
	{
	//	talk("on New Datablock:  Data =" SPC %data.getName());
		Parent::onNewDatablock(%data,%player); //Parent default fcn

		%client = %player.client; //player's client
		if(!isObject(%client)) //if no client exists (e.g. Bot)
			return; //stop here

		//Override Normal Movements (while Adventuring)
	//	commandToClient(%client,'MoveInputOverride',%data.canAdventure); //%data.canAdventureHang);
		commandToClient(%client,'MoveInputOverride',false); //%data.canAdventureHang);
		
		if(%data.getName() $= "PlayerAdventure") //if player data = "PlayerAdventure"
			schedule(1,0,AdventureGlobalSilentLoop,%player); //start the Global Silent Loop
		else if(!%data.canAdventure && %player.lastData.canAdventure) //otherwise if datablock cannot Adventure, and last data can
			AdventureStopHang(%player,true,true); //stop hanging

		%player.lastData = %data; //save past datablock
	}
	//Player on Death
	function Armor::onDisabled(%data,%player,%enabled)
	{
		if(%data.canAdventure) //if datablock is an Adventurer
		{
			%player.client.camera.setMode("Corpse",%player); //restore camera mode
			AdventureStopHang(%player,true); //Stop hang instantly
			AdventureHandHideCheck(%player,"Force"); //Force restore hands
		}
		Parent::onDisabled(%data,%player,%enabled); //Parent default fcn (afterwards)
	}
	//Client on Leave Server
	function GameConnection::OnClientLeaveGame(%client)
	{
		if(isObject(%player = %client.player) && (%data = %player.getDatablock()).canAdventure) //if player exists, and datablock is an Adventurer
		{
			AdventureStopHang(%player,true); //Stop hang instantly
			AdventureHandHideCheck(%player,"Force"); //Force restore hands
		}
		Parent::OnClientLeaveGame(%client); //Parent default fcn (afterwards)
	}
};activatePackage(AdventurePlayer);


//////////
//Aiming//
//////////

//Free Look Check
function serverCmdRelayFreeLook(%client,%val)
{
	%client.FreeLookActive = %val; //save "is Free Looking?" to client (server-side)
}
//First Person Check
function serverCmdRelayFirstPerson(%client,%val)
{
	%client.ThirdPersonActive = %val; //save "is Third Person?" to client (server-side)
}

//Aiming Check (shows Crosshair)
function AdventureAimCheck(%player,%data)
{
	%client = %player.client; //player's client
	%tool = %player.tool[%player.currTool]; //player's current tool (in player's current item slot)
	%show = isObject(%tool) && !%tool.image.melee && !%client.FreeLookActive; //if player has an Item equiped, and the item is not melee, and client is not Free Looking

//	commandToClient(%player.client,'HataCrosshair',%show); //toggle crosshair on/off (depending on %show)

//	if(%data.canAdventureAim && %client.ThirdPersonActive) //if playertype can Aim, and client is in Third Person view, and %show
//		AdventureAimVectorFix(%player,%data); //fix third person Muzzle Vector
}
//Aim Muzzle Vector Correct
function AdventureAimVectorFix(%player,%data)
{
	%mvec = %player.getMuzzleVector(0,true); //muzzle vector
	%fvec = %player.getForwardVector(); //forward vector
	%hack = %player.getHackPosition(); //torso position

	%axis = vectorNormalize(vectorCross(%fvec,"0 0 1"));
	%nvec = matrixMulVector("0 0 0" SPC %axis SPC %data.cameraTilt,%mvec); //tilted muzzle vector
	%sPos = vectorAdd(%hack,"0 0" SPC %data.cameraVerticalOffset); //starting position (full offset)

	%masks = $TypeMasks::PlayerObjectType | $TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::StaticShapeObjectType; //object types to search for
	%hPos = AdventureBeamCheck(%player,%sPos,100,%masks,%nvec,"Aim Check"); //raycast hit position (or beam endpoint if col DNE)
	// if(vectorLen(vectorSub(%hPos,%sPos)) > 4) //if collision point is far enough from the start of the raycast (a bug fix)
		%player.AdvMuzzleEnd = %hPos; //store hit pos to player object
	// else //otherwise
		// %player.AdvMuzzleEnd = ""; //un-define muzzleEnd
}


//////////////////
//Triggered Fcns//
//////////////////

package AdventureTriggers
{
	//Player on Trigger
	function Armor::onTrigger(%data,%player,%slot,%val)
	{
		Parent::onTrigger(%data,%player,%slot,%val);
		if(!%data.canAdventure) //if data cannot Adventure (i.e. player is not in an Adventure Player datablock)
			return; //stop here
		if(%slot == 2 && !%val)
		{
			AdventureGoToLedge(%player,%data);
		//	%nz = getWord(%player.getVelocity(),2);
		//	centerPrint(%player.client,"<font:impact:30><just:center>\c3Air Dive   <color:CCCCCC>nrm z =" SPC %nz,2);
		}
	//	if(%slot == 2 && !%val) //if on Jump, and on Release (of Jump key)
	//		AdventureRestoreMainDatablock(%player,%player.AdvRestoreOnRelease); //restore main datablock, arg. if data. restore delayed because player was 'AdvJumping', the 'AdvRestoreOnRelease' bool = true.
	}
	//Camera on Trigger
	function Observer::onTrigger(%data,%camera,%slot,%val)
	{
		%player = %camera.player; //camera's saved player
		if(isObject(%player) && (%data = %player.getDatablock()).canAdventureHang) //if player exists, and player data. can Hang
		{
			//Hang Jump Trigger (Jump)
			if(%slot == 2 && %val) //if on Jump, and on Press (of Jump key)
				AdventureHangingJump(%player); //Jump along/off the wall
			
			//Hang Cancel Trigger (Crouch)
			if(%slot == 3 && %val && !%client.isCamF8) //if on Crouch, and on Press, and client not F8ing
				AdventureStopHang(%player,false,true); //drop from the ledge (only 1 brick's distance before you can hang again)
			
			//Click or Jet to Exit Orbit Camera (early)
			if((%slot == 0 || %slot == 4) && %val && (%camera.mode $= "Climb" && !%player.isAdventureHanging && !%player.isAdventureClimbTired && !%player.isAdventureSmartDropTired)) //if (Clicking, or Jetting), and (camera mode = "Climb", and not 'Hanging', and not 'Climb Tired', and not 'Smart Drop Tired')
			{
				AdventureStopHang(%player,false,false,false,false,true); //stop hanging, rotate to face cam
				return; //stop here (before parent)
			}
		}
		Parent::onTrigger(%data,%camera,%slot,%val); //Parent default fcn
	}
	//Player on Impact
	function Armor::onImpact(%data,%player,%col,%pos,%speed)
	{
		if(!$Pref::Adventure::FallDamScaling) //if Pref: 'Fall Damage Scaling' off
		{
		//	echo(%player.getShapeName() SPC " On Impact  speed =" SPC %speed);
			if(%speed < PlayerStandardArmor.minImpactSpeed) //if impact speed is less than Standard Player's 'minImpactSpeed'
				%noParent = true; //set 'noParent' bool to true
		}
		else if(%data.canAdventure && ((isObject(%minigame = %player.client.minigame) && %minigame.FallingDamage) || !isObject(%minigame) && $Pref::Server::FallingDamage)) //otherwise if data. is an Adventurer, and ((player client's minigame exists, and minigame Fall-Damage enabled) or (minigame DNE, and Server Fall-Damage enabled))
		{
		//	echo(%player.getShapeName() SPC " On Impact  speed =" SPC %speed);
			%minDamSpeed = $Pref::Adventure::minDamSpeed; //create a shortened local variable for the global Pref: 'Minimum Fall-Damage Speed'
			%minDeathSpeed = $Pref::Adventure::minDeathSpeed; //create a shortened local variable for the global Pref: 'Minimum Fall-Death Speed'
			
			if(%speed < %minDeathSpeed) //if impact speed less than minimum death speed
			{
				%maxDmg = %data.maxDamage; //player datablock's max. damage (equivalent to Max. HP)
				%curDmg = %player.getDamageLevel(); //player's current damage level (higher = more hurt)
				%scale = (%speed-%minDamSpeed) * 10 * (%speed/%minDeathSpeed); //Scale fall damage relative to impact speed (e.g. if speed = 21, minDamSpeed = 20, and minDeathSpeed = 30, then (21-20) * 10 * (21/30) = 7)
				%pain = %curDmg+%scale; //predicted final damage level

			//	talk("speed =" SPC %speed);
			//	talk("pain =" SPC %pain);
			//	talk("scale =" SPC %scale);
				
				if(%pain >= %maxDmg-1) //if pain greater than (max. damage - 1)
					%player.setDamageLevel(%maxDmg-1); //set damage level to max. damage - 1 (and let the onImpact Parent kill them)
				else //otherwise
				{
					if(%speed > %minDamSpeed) //if impact speed greater than Pref: 'Minimum Fall Damage Speed'
						%player.damage(%player,"",%scale,$DamageType::Fall); //hurt the player
					%noParent = true; //set 'noParent' bool to true
				}
			}
		} 
		if(!%noParent) //if 'noParent' bool is false
			Parent::onImpact(%data,%player,%col,%pos,%speed); //Parent default fcn
	}
};activatePackage(AdventureTriggers);


///////////////////
//Pre-Hang Checks//
///////////////////

//Core Climbing Checks
function AdventureClimbCheck(%player,%data)
{
	if(%player.scale !$= "1 1 1") //if player's scale is not standard
		return; //stop here

	%mount = isObject(%player.getObjectMount()); //check if player is mounted to a vehicle
	if(%mount) //if %mount true
	{
		AdventureStopHang(%player,false,false,false,%mount); //stop Hang (%mount arg = true)
		return; //stop here
	}
	
	//Checks for Prone
	%prone = AdventureProneCheck(%player); //Do prone check
	if(%prone) //if player is currently prone
		%player.wasCrouched = true; //store 'wasCrouched' bool to player (check for press of Crouch button)
	else if(%player.AdvLastProne) //otherwise, (if not currently prone, and) if player was prone the last time we checked
		schedule(400,0,eval,%player@".wasCrouched = false;"); //schedule 'wasCrouched' bool reset
	%player.AdvLastProne = %prone; //store last 'prone' bool to player object

	//Check for Floor (Flat-on, and Hang height)
	%isGroundA = isObject(AdventureBoxCheck(%player,false,"","","","Climb Check Floor","1 0 1")); //flat on the ground
	%isGroundB = !%isGroundA ? isObject(AdventureBoxCheck(%player,true,$Adventure::HangHeight,"","","Climb Check Hang","0 1 1")) : %isGroundA; //high enough for Hang ('Hang Height' = 1.3 by default) //if 'ground A' exists, use 'is Ground A' bool
	
	if(!%isGroundB && !%player.isAdventureCancelling) //if high enough to hang, and not climb Cancelling
		AdventureLedgeGrabCheck(%player,%prone); //Do Ledge Grab check
	else if((!%player.isAdventureSmartDropTired && %isGroundA && (%data.canNoJump || %data.canNoMove)) || (%data.canAdventureHang && %isGroundB)) //otherwise if (player flat on the ground, and last ground exists, and (player's datablock can 'No Jump', or player's datablock can 'No Move')), or (player's datablock can Hang, and not high enough for Hang) // || %player.client.camera.isDescentMode //, or camera is in SmartDrop mode
		AdventureStopHang(%player,true); //stop Hang ("true" = %instant arg)
//	else if(%isGroundB && !%player.isAdventureSmartDropTired) //otherwise if not high enough for Hang, and not SmartDrop tired
//		AdventureAirDiveEnd(%player); //End Air Dive early (it missed)
	
	if(%data.canAdventureHang) //if player in Hanging datablock
		AdventureFancyHandCheck(%player,"Silent"); //update hand positions

	//Check for Floor (Step-up height, Smart-Drop height)
//	%isGroundC = !%isGroundA ? AdventureBoxCheck(%player,true,0.6,"","","Climb Check Step","1 0.4 0.5") : %isGroundA; //high enough for Step-up //if 'ground A' exists, use 'is Ground A' bool
	%isGroundD = !%isGroundB ? AdventureBoxCheck(%player,true,3,"","","Climb Check Smart Drop","1 0.5 0.5") : %isGroundB; //high enough for Smart Drop //if 'ground B' exists, use 'is Ground B' bool

	if(%isGroundA) //If flat on the ground
		AdventureLedgeSafety(%player,%data,true,3); //Do ledge walk-off prevention check
	else if(%player.lastGroundAExist) //if last ground exists, and currently high enough for Smart Step
		AdventureSmartDrop(%player,%isGroundD); //Attempt Smart Drop, note Smart Drop min. height as an arg.
	if(%isGroundD) //if not high enough for Smart Drop (high enough for Step)
		AdventureStepUp(%player); //Attempt Step-up

	if(%data.canNoMove) //if midair and datablock is 'NoMove'
		AdventureLedgeSafety(%player,%data); // restore main datablock

	if(%isGroundA) //if ground A exists
		%player.AdvLastPos = %player.getPosition(); //store last position to player
	%player.lastGroundAExist = %isGroundA; //store last ground A found status to player
}

function AdventureProneCheck(%player)
{	
	//Positions
	%pos = %player.getPosition(); //player's base position (under feet)
	%hack = %player.getHackPosition(); //player's torso position
	%prone = mAbs(getWord(%hack,2)-getWord(%pos,2)) < 0.6; //if player is not fully crouched (Hack's z component minus Pos's z component is greater than 0.7)
//	talk("Prone Check:  Dist =" SPC (getWord(%hack,2)-getWord(%pos,2)));
	return %prone; //return 'prone' bool
}

//Ledge Grab
function AdventureLedgeGrabCheck(%player,%prone)
{
	//Positions
	%pos = %player.getPosition(); //player's base position (under feet)
	if(!%prone) //if player is not fully prone
		%posA = vectorAdd(%pos,"0 0 1.325"); //Imitation Hack Position (unaffected by crouching animation)
	else //otherwise
		return; //stop here

	//Initial Velocity
	%vel = %player.getVelocity(); //player's velocity

	//Arm Flailing
	%FallDamageOn = (isObject(%minigame = %player.client.minigame) && %minigame.FallingDamage) || (!isObject(%minigame) && $Pref::Server::FallingDamage); //(player client's minigame exists, and minigame Fall-Damage enabled) or (minigame DNE, and Server Fall-Damage enabled)
	if(%FallDamageOn) //if falling speed greater than Pref: 'Minimum Fall-Damage Speed', and Falling Damage on
	{
		%upv = getWord(%vel,2); //vertical component of player velocity
		%speed = -%upv; //define speed as inverted vertical velocity (so it's positive if falling)
		%minDamSpeed = $Pref::Adventure::minDamSpeed; //create a shortened local variable for the global Pref: 'Minimum Fall-Damage Speed'
		%minDeathSpeed = $Pref::Adventure::FallDamScaling ? $Pref::Adventure::minDeathSpeed : PlayerStandardArmor.minImpactSpeed; //if Pref: 'Fall Damage Scaling' on - use Pref: 'Minimum Fall-Death Speed', else - use Standard Player's 'minImpactSpeed'
		%atMinDeathSpeed = %speed > %minDeathSpeed; //if falling speed greater than Pref: 'Minimum Fall-Death Speed'
		if($Pref::Adventure::FallDamScaling || %atMinDeathSpeed) //if Pref: 'Fall Damage Scaling' on, or 'at min. Death speed' = true
			AdventureArmFlailAnim(%player,%speed,%minDamSpeed,%minDeathSpeed); //Do Arm Flail at unsafe fall speeds
		if(%atMinDeathSpeed) //if at min. Death speed = true
			return; //stop here (don't check for ledges)
	}

	//Climbing exceptions
	if((%player.client.KeyActionCrouch && !%player.isAdventureSmartDropTired) || %player.isAdventurePullup || %player.isAdventureClimbing || %player.isAdventureHangJumping) //if player is (crouching and Smart-Drop timed out), or Pulling-up, or climbing, or hang jumping
		return; //stop here

	//Ledge Searching
	%len = $Adventure::LedgeHangLen; //ideal length from player to brick
	%vec = %player.getForwardVector(); //player forward vector
	%dir = HataDirCheck(%vec); //calculate cardinal directional vector from forward vector
	%rot = vectorToAxis(%dir); //convert dir to axis rotation

	%isLedge = AdventureLedgeSearch(%player,%posA,"",%dir); //Check for ledge
	if(!%isLedge) //if ledge not found
	{
		%firstHang = %player.isAdventureFirstHang; //true if player is already 'Hanging'
		%lim = !%firstHang ? 10 : 2; //max plate range for Multi Ledge Check limit
		%lim = %player.isAdventureHangJumpTired ? 6 : %lim; //max plate range for Multi Ledge Check limit
		%mPosA = AdventureMultiLedgeCheck(%player,%posA,%dir,%firstHang,%lim); //Check for ledge within a 10-plate vertical range, args. - ledge check starting position, search directional vector, center the search if true, plate range = 'lim'
		if(%mPosA !$= "") //if Multi-Ledge check found something
			%snapPos = AdventureVerticalSnap(%player,%mPosA,%posA,%lim); //define %snapPos
		else //otherwise
			AdventureAirDiveCheck(%player,%posA); //Attempt Air Dive
		if(%snapPos !$= "") //if snap Position defined successfully
		{
		//	talk("Ledge Grab Check:  SnapPos found:  Limit =" SPC %lim);
			%pos = %snapPos; //update pos to vertical Snap'd Position
			%posA = vectorAdd(%pos,"0 0 1.325"); //update posA relative to Snapped position
			%isLedge = true; //manually set 'isLedge' bool to true (we did find a ledge, to snap to)
		}
	}
	if(%isLedge) //if ledge exists, or Snap Position exists
		AdventureLedgeHang(%player,%posA,%pos,%rot,%dir,%vel); //Do actual ledge hang loop
}


/////////////////
//Start Hanging//
/////////////////

//Do Hang Actual
function AdventureLedgeHang(%player,%posA,%pos,%nRot,%dir,%vel)
{
	//General
	%client = %player.client; //player's client
	%camera = %client.camera; //client's camera
	%dist = PlayerAdventureHang.cameraMaxDist; //player data.'s camera distance (use same value for orbit cam)
	%rvec = findRelativeX(%dir); //rightward vector
	%lvec = vectorScale(%rvec,-1); //leftward vector
	
	//Keep Camera Location if already Climbing
	if(%camera.mode $= "Climb" || %camera.isDescentMode) //If camera mode = "Climb", or camera is in Descent Mode
		%vector = %camera.getForwardVector(); //Camera's look vector (Note: I modified the default camera 'getForwardVector' fcn)
	else //otherwise
		%vector = %player.getEyeVector(); //player's eye vector (influenced by Free Look)

	%z = getWord(%vector,2); //z axis (up/down)
	%camXY = getWords(%vector,0,1) SPC 0;
	%camXYN = vectorNormalize(%camXY);

	%hack = %player.getHackPosition(); //Free Look Cam
	%back = vectorScale(%dir,-1); //
	%pivt = vectorAdd(%hack,%back); //
	%pivt = vectorAdd(%pivt,"0 0 0.5");
	%tra = %camera.getTransform(); //

	%groundB = AdventureBoxCheck(%player,true,$Adventure::HangHeight,%pos,"","Ledge Hang - Hang","0 1 1"); //make sure position is high enough for Hang, after the Vertical Snap
//	talk("Player Cam F8? =" SPC %client.isCamF8);
//	talk("Player Adventure Hang Bool =" SPC %player.isAdventureHanging);
	if(isObject(%groundB) || (%client.isCamF8 && !%player.isAdventureHanging))
		return; //stop here

	%upv = getWord(%vel,2); //vertical component of player velocity
	%speed = -%upv; //define speed as inverted vertical velocity (so it'll be positive if falling)
	
	%minDamSpeed = $Pref::Adventure::minDamSpeed; //create a shortened local variable for the global Pref: 'Minimum Fall-Damage Speed'
	if(%speed > %minDamSpeed && !%player.isAdventureFirstHang) //if falling speed greater than Pref: 'Minimum Fall-Damage Speed'
	{
		if($Pref::Adventure::FallDamScaling) //if Pref: 'Fall Damage Scaling' on, or 'at min. Death speed' = true
		{
			%minDeathSpeed = $Pref::Adventure::minDeathSpeed;  //create a shortened local variable for the global Pref: 'Minimum Fall-Death Speed'
			
			%maxDmg = PlayerAdventure.maxDamage;
			%curDmg = %player.getDamageLevel();
			%scale = (%speed-%minDamSpeed) * 10 * (%speed/%minDeathSpeed) / 2; //Scale fall damage relative to impact speed (and halve it)
			%pain = %curDmg+%scale; //predicted final damage level

		//	talk("up vel =" SPC %speed);
		//	talk("pain =" SPC %pain);
		//	talk("scale =" SPC %scale);
			if(%speed > %minDamSpeed) //if impact speed greater than Pref: 'Minimum Fall Damage Speed'
				%player.damage(%player,"",%scale,$DamageType::Fall); //hurt the player
			if(%player.getState() $= "Dead") //if the damage killed him
				return;
		}
	}
	
	//Edge Detection
	%ledgeL = AdventureEdgeDetect(%player,%posA,%lvec,"ledge","L"); //If leftward ledge found (can move Left)
	%ledgeR = AdventureEdgeDetect(%player,%posA,%rvec,"ledge","R"); //If rightward ledge found (can move Right)
	%blockL = AdventureEdgeDetect(%player,%posA,%lvec,"block","L"); //If leftward perpendicular wall found (cannot move Left)
	%blockR = AdventureEdgeDetect(%player,%posA,%rvec,"block","R"); //If rightward perpendicular wall found (cannot move Right)

	//Can Move Left/Right
	%canMovL = %ledgeL && !%blockL; //if leftward ledge found, and leftward block not found
	%canMovR = %ledgeR && !%blockR; //if rightward ledge found, and rightward block not found
	
	//Keyboard inputs - Leftward (relative)
	%inputLeft = MoveInputCamSync(%client,"left",%vector,%dir,0.5); //if input - left
	%svec = %inputLeft ? %lvec : ""; //if 'inputLeft' bool = true - use 'leftward vector' for 'side vector', else - undefine it

	//Keyboard inputs - Rightward (relative)
	%inputRight = MoveInputCamSync(%client,"right",%vector,%dir,0.5); //if input - right
	%svec = %inputRight ? %rvec : %svec; //if 'inputRight' bool = true - use 'rightward vector' for 'side vector', else - keep previous

	//First Hang (Per Climbing Session)
	if(!%player.isAdventureFirstHang) //if 'First Hang' bool is false (very first hang)
	{
		%player.isAdventureFirstHang = true; //set 'First Hang' bool to true
		%player.isAdventurePreHang = true; //set 'Pre-Hang' bool to true
		cancel(%player.AdvDataRestore); //cancel Stop Hang finish sched
		cancel(%player.AdvPreHang); //cancel Pre-Hang sched
		%time = %camera.mode $= "Climb" ? 1 : 100; //if camera mode is "Climb" - use 1, else - use 100
		%player.AdvPreHang = schedule(%time,0,eval,%player@".isAdventurePreHang = false;"); //wait 'time' ms, then set 'Pre Hang' to false
	}

	//Move & rotate player, spawn shape
	%player.setTransform(%pos SPC %nRot); //combines all adjustments
	AdventurePreSpawnClimbTool(%player,%pos,%dir,50); //create the platform, args. - shape starting position, vector for rotation, shape self-delete timeout value

	//Inital Grab Code (per ledge)
	if(!%player.isAdventureHanging) //First climb check
	{
	//	talk("Hang:  on Ledge Grab");
	//	talk("Hang:  Climb Timb =" SPC (getSimTime()-%player.lastSimTime));
		if(%upv > 0 && %player.isAdventurePreHang) //if vertical velocity greater than 2, and 'Pre-Hang' bool is true
			AdventurePreSpawnClimbTool(%player,%posA,%dir,50,"up"); //Spawn a shape directly above the player (to nullify upward velocity cleanly), args. - starting position, vector for rotation, self-delete timeout value (must be 50+), name suffix
		
		%player.isAdventureHanging = true; //set 'Hanging' bool to true
		%player.setVelocity("0 0" SPC %upv); //reset hanging velocity (-5 fixes a glitch where the player falls off immediately)
		%player.changedatablock(PlayerAdventureHang); //change to climbing datablock
		%player.hangCol.onLedgeGrab(%client); //Event hook for "onLedgeGrab"
		
		if(%camera.mode !$= "Climb") //if camera's mode is not "Climb"
		{
		//	%tra = %player.getEyeTransform(); //player's eye transform (matches head)
			%camera.player = %player; //define camera's player
			%camera.mode = "Climb"; //set Camera mode to "Climb"
			%player.hasCamSetUp = false; //reset this bool so the camera will update
		}
		
		%player.isAdventureHangAnim = false; //reset 'Hang Anim' bool to false
		%player.isAdventureHangAnimReady = false; //reset 'Hang Anim Ready' bool to false
		cancel(%player.AdvHangAnim); //cancel any previous schedules
		%player.AdvHangAnim = schedule(50,0,eval,%player@".isAdventureHangAnimReady = true;"); //wait 50 ms, then set 'Hang Anim Ready' bool to true
	}
	
	//Camera - Orbit Mode
	%df = vectorDist(%dir,%camXYN); //distance between forward dir. vector, and horizontal camera vector
	if((%df < 1.6 || %z > 0.9 || %z < -0.9) && !%player.hasCamSetUp) //if (df less than 1.6, or z less than 0.9) AND player does not have CamSetUp
	{
		if(!%player.isAdventureAirDive && (!%player.isAdventureSmartDropTired || %player.isCamPointMode))
		{
			%cRot = vectorToAxis(%vector); //make axis rotation from look 'Vector'
			%camera.setOrbitMode(%player,"0 0 0" SPC %cRot,0,%dist,%dist,true); //Normal climb cam
			%camera.setTransform("0 0 0" SPC %cRot); //update camera (position and) rotation
		}
		%player.hasCamSetup = true; //set 'hasCamSetup' bool to true
		%player.isCamPointMode = false; //set 'isCamPointMode' bool to false
	}
	//Camera - Point Mode (Backward Free Look)
	else if((%df > 1.6 && %z < 0.9 && %z > -0.9) && !%player.isCamPointMode && %player.hasCamSetup)
	{
		%camera.setOrbitPointMode(%pivt,0); //Backwards facing climb cam
		%player.isCamPointMode = true; //set 'isCamPointMode' bool to true
		%player.hasCamSetup = false; //set 'hasCamSetup' bool to false
	}

	//Control Object Fixes
	if(%camera.getControlObject() != %camera)
		%camera.setControlObject(%camera); //confirm camera is controlling itself
	if(%client.getControlObject() != %camera)
		%client.setControlObject(%camera); //confirm client is controlling camera

	//Movement
	if(%player.isAdventureFirstHang && !%player.isAdventurePreHang && !%player.isAdventureClimbTired && !%player.isCamPointMode && !%client.isCamF8 && !%player.isAdventurePullup) //if 'First Hang' true, and 'Pre-Hang' false, and 'Climbing Tired' false, and 'Point Mode' false, and 'Cam F8ing' false, and 'Pulling Up' false
	{
		//Event integration
		%player.hangCol.onLedgeMove(%client); //use currently grabbed brick

		//Vertical climbing
		%hiPivot = -0.75; //point where %up is n/a
		%loPivot = -0.85; //point where %up and %down flip
		%forward = MoveInputCamSync(%client,"forward",%vector,%dir,0.9); //if input - forward
		%backward = MoveInputCamSync(%client,"backward",%vector,%dir,0.9); //if input - backward

		%backDown = (%client.keyMoveForward && %df > 1.6 && %player.hasCamSetup);
		%backUp = (%client.keyMoveBackward && %df > 1.6 && %player.hasCamSetup);

		%up = (%forward && %z > %hiPivot || %backward && %z < %loPivot || %backUp) && !%backDown; //if (input - forward, and looking up) or (input - backward, and looking down)
		%down = (%backward && %z > %hiPivot || %forward && %z < %loPivot || %backDown) && !%backUp; //if (input - backward, and looking up) or (input - forward, and looking down)
		
		%up = !$Pref::Adventure::FluidTraverse ? %forward : %up; //if Pref: Fluid-Traversal off - use %forward, otherwise - keep %up
		%down = !$Pref::Adventure::FluidTraverse ? %backward : %down; //if Pref: Fluid-Traversal off - use %backward, otherwise - keep %down

		if(%up)
			AdventureClimbUp(%player,%svec); //Climb Up
		if(%down)
			AdventureClimbDown(%player,%svec); //Climb Down

	//	centerPrint(%client,"<font:impact:30><just:center>\c3" @ %player.getShapeName() SPC "\c6  Back? =" SPC %backward SPC "  Forward? =" SPC %forward SPC "  Left? =" SPC %inputLeft SPC "  right =" SPC (%inputRight && %canMovR),5);

		//Horizontal climbing
		%xMov = vectorScale(%svec,4);
		if((%canMovL && %inputLeft || %canMovR && %inputRight) && !%player.isAdventureClimbing) //add horizontal velocity, if not climbing up/down already
		{
			%dot = vectorDot(%vel,%svec);
			if(%dot < 4 || %dot > 5) //leave some leeway for velocity to continue
				%player.setVelocity(%xMov); //set velocity leftward
			%canMove = true; //note: we can Move (for Fluid Traversal fancy hand fix)
		}
		
		//Fluid Traversal
		if(%client.KeyMoveForward && (%z < 0.8 && %z > -0.8) && !%player.isAdventureClimbing) //if holding W, and , and not looking too far up or down, and not climbing
			%travHand = AdventureFluidTraversal(%player,%camera,%canMove); //attempt Free Climbing (and return true if hand shows)
	//	if(!%travHand && %player.AdvHangAnimSide $= "fancy") //otherwise, if 'Hang Anim Side' string = "fancy"
	//		%player.AdvHangAnimSide = ""; //clear 'Hang Anim Side' string
	}
	
//	talk("Hang:  z =" SPC %z);
	//Play Hanging Animations
	AdventureUpdateHangAnim(%player,%player.isAdventureHangAnimReady,%ledgeL,%ledgeR,%blockL,%blockR,%travHand); //args. - starting, can L, can R, block L, block R
}


////////////////
//Stop Hanging//
////////////////

//Stop Hanging
function AdventureStopHang(%player,%instant,%force,%cam,%mount,%turn)
{
	if(!isObject(%player) || %player.getState() $= "Dead") //if player object DNE, or player is dead
		return; //stop here

//	talk("Adventure hang stop called by " SPC %player.getShapeName() SPC "  instant =" SPC %instant SPC "  force =" SPC %force SPC "  cam =" SPC %cam SPC "  mount =" SPC %mount);
	%client = %player.client; //define client
	%camera = %client.camera; //define camera
	%data = %player.getDatablock(); //define player datablock

//	if(%instant && %data.canAdventureCrouch) //if stop called by silent loop, and data 'canCrouch' (override for No-Jump running)
//		return; //stop here

	//If hanging, don't let hang cam overwrite previous f7
	if(%cam && !%client.isCamF8 && %camera.mode !$= "Corpse" && %camera.AdvLastTrans !$= %camera.getTransform()) //if 'Stop Hang' called by a Camera fcn, and player is not F8ing, and camera's mode is not "Corpse", and camera's old Transform is not the current transform
		%camera.setTransform(%camera.AdvLastTrans); //Teleport Camera to saved Transform (i.e. world position and rotation)
	if(%cam && !%player.isAdventureHanging) //if 'Stop Hang' called by a Camera fcn, and 'AdventureHanging' bool is false
		return; //stop here

//	cancel(%player.AdvHang); //cancel 'is Hanging' schedule
	cancel(%player.AdvPreHang); //cancel 'Pre-hang' schedule
	cancel(%player.AdvClimbing); //cancel 'is Climbing' schedule

	%player.hasCamSetup = false; //note: cam not set up (for point mode fix)
	%player.isAdventureHanging = false; //note: not hanging
	%player.isAdventureClimbing = false; //note: not climbing
	%player.isAdventureFirstHang = false; //note: not in climb mode

	if(%force || %instant && %data.canAdventureHang) //if force = true, or (instant = true, and player's datablock can Hang)
	{
		%player.isAdventureCancelling = true; //Disable climbing
		schedule(200,0,eval,%player@".isAdventureCancelling = false;"); //un-disable it after 200ms

		//Fancy Cam Bonus
		%fwdVec = %player.getForwardVector(); //forward vector pre transform
		%camVec = %camera.getForwardVector(); //camera's look vector (Note: I modified the default camera 'getForwardVector' fcn)
		%xy0 = getWords(%camVec,0,1) SPC 0; //horizontal cam vec
		%xyN = vectorNormalize(%xy0); //normalized hori. cam vec

		%z = getWord(%cvc,2); //vertical cam vec component
		%df = vectorDist(%fwdVec,%xyN); //distance from forward vector
		%pivot = 0.3; //absolute z value where down / up becomes true
		
		%up = %z > %pivot; //true if - looking moderately diagonally up
		%down = %z < -%pivot; //true if - looking moderately diagonally down
		%drop = %z < $Adventure::SmartDropZ; //true if - looking far diagonally down
		if((%up || %down || %drop) && $Pref::Adventure::SmartDrop) //if ('up', or 'down', or 'drop'), and Pref: Smart-Drop On
			%fancy = true; //Smart Drop finish
		
		%turn = %df > 0.5; //if 'distance from forward vec' is greater than 1, turn = true
	//	%cam = !%turn; //if 'turn' = false, set cam to true
	}
	if(%fancy) //if Smart Drop (%fancy = Smart Drop)
	{
	//	talk("Stop Hang:  z =" SPC %z);
		// %newData = %up ? "PlayerAdventureLookUp" : ""; //if 'up' = true - use 'Look Up' data., else - leave it undefined
		// %newData = %down ? "PlayerAdventureLookDown" : %newData; //if 'down' = true - use 'Look Down' data., else - keep last 'newData'
		// %newData = %drop && !%player.isAdventurePullup ? "PlayerAdventureSmartDrop" : %newData; //if 'drop' = true - use 'Smart Drop' data., else - keep last 'newData'
		// %player.changedatablock(%newData); //Force player to look diagonally downward / upward (to match Camera direction)
		if(!%player.isAdventurePullup && (%down || %drop)) //if not Pulling-up, and ('down' = true, or 'drop' = true)
		{
			if(%drop) //if Smart Drop ready
			{
			//	cancel(%player.AdvSmartDropTired); //cancel 'SmartDropTired' reset schedule
			//	%player.isAdventureSmartDropTired = false; //reset 'SmartDropTired' to false immediately
			}
			
			%vXY = getWords(%player.getVelocity(),0,1); //horizontal velocity components
			%dv = -5*(mAbs(%z)-%pivot)*1/(1-%pivot); //scale downward velocity add, relative z component of look vector
			%player.setVelocity(%vXY SPC %dv); //add downward velocity (looks & feels better)
		//	talk("Stop Hang:  Down Vel. Add =" SPC %dv);
		}
	}
	if(%turn) //if 'Turn-to-Match Camera Look Direction' bool is true
	{
		%pos = %player.getPosition(); //player position
		%rot = vectorToAxis(%camera.getForwardVector()); //Use vertical vector-fixed rotation
		%player.setTransform(%pos SPC %rot); //Transform player object (set its position and rotation)
	}
//	talk("Data =" SPC %data.getName());
	if(%data.canAdventureHang) //|| %fancy)
	{
		// if(%fancy) //if Smart Dropping
			// %player.schedule(35,changedatablock,PlayerAdventureNoAirCtrl); //The 'SmartDrop' camera tilt does not work if time < 35.
		// else //otherwise
		%player.changedatablock(PlayerAdventureNoAirCtrl); //swap to 'NoAirCtrl' datablock immediately
	//	%player.changedatablock(PlayerAdventureNoMove); //swap to 'NoAirCtrl' datablock immediately

		cancel(%player.AdvDataRestore); //cancel any pending 'DataRestore' schedules
		%player.AdvDataRestore = schedule(400,0,AdventureRestoreMainDatablock,%player); //wait 400ms, then restore 'AdventurePlayer' data.

		AdventureStopHangAnim(%player,%mount); //Reset animations
	//	talk("datablock change in StopHang called");
	}
	else if(%instant) //if called via silent loop
	{
		cancel(%player.AdvDataRestore); //cancel any pending 'DataRestore' schedules
		AdventureRestoreMainDatablock(%player); //immediately, restore restore 'AdventurePlayer' datablock
	}

//	commandToClient(%client,'hataForceAim',%camVec,%fwdVec); //set player look vector to match camera look vector

	if(!%cam && !%player.isAdventureAirDive) //if 'Stop Hang' not called by a Camera fcn, and 'AirDive' bool = false
	//	schedule(1,0,AdventureCamRestore,%player,%camVec); //wait 1ms, then call 'Cam Restore' fcn (the delay hides a jarring camera change)
		AdventureCamRestore(%player,%camVec,%fwdVec); //wait 1ms, then call 'Cam Restore' fcn (the delay hides a jarring camera change)

	AdventureHandHideCheck(%player,"Force"); //Force restore hands
	HataShapeDelete(%player.client.AdvTool); //insta-delete Climb Tool
	HataShapeDelete(%player.client.AdvShow); //insta-delete Climb Show Tool (the blue shape visible during debug)
}


////////////
//Climb Up//
////////////

//Climb Upwards (brick by brick)
function AdventureClimbUp(%player,%svec)
{
	%client = %player.client; //define client
	if(!%player.client.KeyActionJump) //if player is not Pressing the Jump key
	{
		%hack = %player.getHackPosition(); //torso
		%vec = %player.getForwardVector(); //forward vector
		%res = AdventureClimbLedgeFind(%player,%hack,%vec,%svec,false); //result of a search for ledge within climb reach
		%vel = getWords(%res,0,2); //final velocity add
		%time = getWord(%res,3); //z add scale
		%rPos = getWords(%res,4,6); //relative hack pos
	}
	if(%res !$= "") //if Climb Ledge Find returned words
	{
	//	centerPrint(findclientbyname(),"<font:impact:30><just:center>\c5Climb Up   <color:CCCCCC>time =" SPC %time @ "<br>vel =" SPC %vel,10);

		cancel(%player.AdvClimbTired);
		%player.isAdventureClimbing = true;
		%player.isAdventureClimbTired = true;
		%player.AdvHang = schedule(%time,0,eval,%player@".isAdventureHanging = false;");
		%player.AdvClimbing = schedule(%time,0,eval,%player@".isAdventureClimbing = false;");
		%player.AdvClimbTired = schedule(%time*1.1,0,eval,%player@".isAdventureClimbTired = false;"); //was *1.3
		AdventurePreSpawnClimbTool(%player,%rPos,%vec,%time,"up"); //spawn shape ahead of time, args. - shape starting position, vector for rotation, shape self-delete timeout value (-1 means it won't delete itself automatically)
		%player.setVelocity(%vel); //update player velocity
		AdventureClimbAnimCheck(%player,%time,%rPos,%svec); //do upward climbing animation
	}
	else if(%player.client.KeyActionJump) //if holding Space
		AdventurePullup(%player); //Attempt Pullup
}


//////////////
//Climb Down//
//////////////

//Climb Downwards (brick by brick)
function AdventureClimbDown(%player,%svec)
{
	if(!%player.client.KeyActionJump) //if player is not Pressing the Jump key
	{
		%hack = %player.getHackPosition(); //torso
		%vec = %player.getForwardVector(); //forward vector
		%res = AdventureClimbLedgeFind(%player,%hack,%vec,%svec,true); //result of a search for ledge within climb reach
		%vel = getWords(%res,0,2); //final velocity add
		%time = getWord(%res,3); //Climb time
		%rPos = getWords(%res,4,6); //relative hack pos
		%fPos = vectorAdd(%rPos,"0 0" SPC -1.325); //predicted player position (after Climbing Down)
	}
	if(%res !$= "") //if Climb Ledge Find returned words
	{
	//	centerPrint(findclientbyname(),"<font:impact:30><just:center>\c3Climb Down   <color:FFFFFF>time =" SPC %time SPC "  res =" SPC %res SPC "  vel =" SPC %vel,10);

	//	talk("Climb Down:  Result =" SPC %res);
		cancel(%player.AdvClimbTired);
		%player.isAdventureClimbing = true;
		%player.isAdventureClimbTired = true;
		%player.AdvHang = schedule(%time,0,eval,%player@".isAdventureHanging = false;");
		%player.AdvClimbing = schedule(%time,0,eval,%player@".isAdventureClimbing = false;");
		%player.AdvClimbTired = schedule(%time*1.1,0,eval,%player@".isAdventureClimbTired = false;"); //was *1.6
		AdventurePreSpawnClimbTool(%player,%fPos,%vec,-1); //spawn shape ahead of time, args. - shape starting position, vector for rotation, shape self-delete timeout value (-1 means it won't delete itself automatically)
		%player.setVelocity(%vel);
		AdventureClimbAnimCheck(%player,%time,%rPos,%svec,true); //do downward climbing animation
	}
}


/////////////////
//Ledge Pull-up//
/////////////////

//Pull-up (onto top Ledge)
function AdventurePullup(%player)
{
	%client = %player.client; //client
	%camVec = %client.camera.getForwardVector(); //camera vector
	%vec = %player.getForwardVector(); //forward vector (ledge-facing)

	%left = MoveInputCamSync(%client,"left",%camVec,%vec,0.5); //if input - left
	%right = MoveInputCamSync(%client,"right",%camVec,%vec,0.5); //if input - right
	%backward = MoveInputCamSync(%client,"backward",%camVec,%vec,0.5); //if input - backward

	if(%left || %right || %player.isAdventureHangJumping || %player.isAdventurePreHang || !%player.isAdventureHanging || %player.isAdventurePullup) //if input - left, right or back, or wall jumping, or not Anim, or Pulling-up already
		return; //stop here // || %backward

	%hack = %player.getHackPosition(); //torso position
	%len = $Adventure::LedgeHangLen; //Ideal ledge length (global var.)
	%mov = vectorAdd(%hack,vectorScale(%vec,%len-0.4)); //Equals 0.3 if Ideal length is 0.7
	%mov = vectorAdd(%mov,"0 0" SPC 1.5);
	%block = AdventureBoxCheck(%player,false,2.5,%mov,"","Pullup","1 0.5 1"); //2.6 is the exact height of 5 bricks
	if(isObject(%block)) //if Pullup successful
		return; //stop here

//	talk("Pulled-up   Timing? =" SPC %timing);
	if(%player.isCamPointMode) //if free looking
	{
		%camera = %client.camera; //client's camera
	//	%tra = %camera.getTransform(); //camera's current transform
		%rot = vectorToAxis(%camera.getForwardVector()); //Use fixed rotation
		%player.hasCamSetup = false; //reset orbit cam
		%camera.setOrbitMode(%player,"0 0 0" SPC %rot,0,PlayerAdventureHang.cameraMaxDist,0,true); //Normal climb cam
		%camera.setTransform("0 0 0" SPC %rot); //rotate orbit cam to match old view
	}
	%fwdAdd = vectorScale(%vec,4); //was 4
	%player.setVelocity("0 0" SPC 10); //10.5
	
	%player.playthread(0,plant);
	%player.playthread(3,shiftdown,true);
	
	%time = 280; //timeout before Pullup forward velocity add
//	schedule(50,0,AdventurePullupAnim,%player); //wait 50 ms, then call 'Pullup Animation' fcn (inverted Crouch)
	AdventurePullupAnim(%player); //wait 50 ms, then call 'Pullup Animation' fcn (inverted Crouch)
	schedule(%time,0,AdventurePullupFinish,%player,(getWords(%fwdAdd,0,1) SPC 0)); //wait 'time' ms, then set forward velocity

	%player.isAdventurePullup = true;
	schedule(350,0,eval,%player@".isAdventurePullup = false;");
}
function AdventurePullupAnim(%player)
{
	%player.playthread(2,crouch,true);
	%player.schedule(200,setThreadDir,2,false);
}
function AdventurePullupFinish(%player,%final)
{
//	AdventurePullupAnim(%player); //wait 50 ms, then call 'Pullup Animation' fcn (inverted Crouch)
//	talk("Pullup Finale called");
	%player.setVelocity(%final);
}


////////////////
//Hanging Jump//
////////////////

//Adventure Hang Jump
function AdventureHangingJump(%player)
{
//	echo("Adventure Hang Jump called");
	%client = %player.client; //player's client
	if(%player.isAdventureHangJumpingTired || !%player.isAdventureHanging || %player.isAdventureClimbTired || %player.isAdventurePreHang || %player.isAdventurePullup || %player.isAdventureAirDive || !$Pref::Adventure::HangingJump) //if player is Hang Jump Timed-out, or not hanging, or pre-hang, or Pulling-up, or Air-Diving, or Pref: 'Hanging-Jump' off
		return; //stop here

	//Camera
	%camera = %client.camera; //client's camera
	%camTra = %camera.getTransform(); //camera transform
	%camRot = getWords(%camTra,3,6); //Get camera rotation from camera transform

	//Player
	%pos = %player.getPosition(); //Player position
	%vec = %player.getForwardVector(); //Player forward vector
	%dir = HataDirCheck(%vec); //wall-facing vector
	%rvc = findRelativeX(%dir); //rightward vector

	//Free Look
	if(%player.isCamPointMode) //if player is free looking
	{
		%back = vectorScale(%dir,-1); //backward offs
		%pos = vectorAdd(%pos,%back); //use free look offset'd position
	}

	%cvc = %camera.getForwardVector(); //camera's look vector (Note: I modified the default camera 'getForwardVector' fcn)
	%cxyN = vectorNormalize(getWords(%cvc,0,1) SPC 0); //make vector from horizontal components of cam vec, and normalize it

	//Client Movement Inputs
	%right = MoveInputCamSync(%client,"right",%cxyN,%vec,0.6); //if input - right
	%left = MoveInputCamSync(%client,"left",%cxyN,%vec,0.6); //if input - left
	%backward = MoveInputCamSync(%client,"backward",%cxyN,%vec,0.9); //if input - backward

	%df = vectorDist(%dir,%cxyN); //forward

	if((%client.KeyMoveForward && %df > 1.3) && (!%client.KeyMoveBackward && !%client.KeyMoveRight && !%client.KeyMoveLeft)) //if holding W, and not holding any other keys
	{
		%side = 4;
		%input++; //
	//	talk("Distance from Forward =" SPC %df);
	}
	else
	{
		if(%right && !%player.isCamPointMode) //if input - right, and not in back cam
		{
			%side = 1;
			%input++; //count the number of inputs
		}
		if(%left && !%player.isCamPointMode) //if input - left, and not in back cam
		{
			%side = 2;
			%input++; //
		}
		if(%backward && !%player.isCamPointMode) //if input - backward, and not in back cam
		{
		//	%side = 3;
			%input++; //
		}
	}

	if(%side > 0 && %input == 1) //if %side is valid, and only 1 key is pressed
	{
	//	if(%side == 4 && %df < 1) //if holding W, and looking towards the wall (a fix to allow Pulling-up while holding W)
	//		return; //stop here

		if(%side == 1) //rightward jump
		{
			%dirVec = %rvc;
			%player.playthread(2,shiftaway,true); //override a slot 2 animation in while climbing
		}
		
		if(%side == 2) //leftward jump
		{
			%dirVec = vectorScale(%rvc,-1);
			%player.playthread(2,leftrecoil,true); //override a slot 2 animation in while climbing
		}
		
		if(%side == 3) //backward jump
		{
			%dirVec = vectorScale(%dir,-1);
			%rot = vectorToAxis(%dirVec); //rot from vector
			%player.setTransform(%pos SPC %rot);
			%player.playthread(2,shiftup,true); //override a slot 2 animation in while climbing
		}

		if(%side == 4) //Point Mode aimed jump
		{
			AdventureStopHang(%player);
			%player.setTransform(%pos SPC %camRot);
			%dirVec = %player.getForwardVector(); //use forward vector after transform
			%player.playthread(2,shiftdown,true); //override a slot 2 animation in while climbing
		}
		else //otherwise (if not a Point Mode aimed jump)
		{
			%player.isAdventureHanging = false;
			%player.isAdventureHangJumping = true;
			schedule(300,0,eval,%player@".isAdventureHangJumping = false;");
		}

		%player.playthread(0,jump);
		serverPlay3D(jumpsound,%pos);

		%player.isAdventureHangJumpingTired = true;
		cancel(%player.AdvHangJumpTired);
		%player.AdvHangJumpTired = schedule(400,0,eval,%player@".isAdventureHangJumpingTired = false;");
	}
	else //otherwise
		return; //stop here

//	centerPrint(%player.client,"<font:impact:30><just:center>\c5Hang Jump\c5  successful",2);

	%zA = "0 0" SPC 3; //normal z add (was 4)
	%yA = vectorScale(%dirVec,10); //(was 10)
	%yzA = vectorAdd(%yA,%zA);
	%player.setVelocity(%yzA);
}


////////////////
//Ledge Safety//
////////////////

//Prevent Player from Walking off a High Ledge
function AdventureLedgeSafety(%player,%data,%low,%height)
{
	if(!$Pref::Adventure::LedgeSafety) //if Pref: 'Ledge Safety' off
		return; //stop here
	if(%data.canNoMove && (%player.client.KeyActionCrouch || %player.client.KeyActionJump || !$Pref::Adventure::LedgeSafety)) //if data. can 'NoMove', and (player is Pressing Crouch, or player is Pressing the Jump key, or Pref: 'Ledge Safety' off)
	{
		AdventureLedgeSafetyFinish(%player,%player.client.KeyActionJump); //Restore datablock and clear saved vectors
		return; //stop here
	}
	if(%player.wasCrouched || !$Pref::Adventure::LedgeSafety) //if was-Crouched bool = true (if still Holding, or Released Crouch less than 400 ms ago, or Pref: 'Ledge Safety' off)
		return; //stop here

	%vel = %data.canNoMove ? %player.AdvLedgeSafeVec[2] : %player.getVelocity();

	%mvec = %player.getMuzzleVector(0,true);
	%fvec = %player.getForwardVector(); //player forward vec
	%dot = vectorDot(%vel,%fvec); //compare vel to dir
	%front = %dot > 0.4; //if moving forward (was 0.2)

	if(%front && (%z = getWord(%mvec,2)) < -0.95) //if SmartDrop Ready
	{
		if(%data.canNoMove)
			AdventureLedgeSafetyFinish(%player); //Restore datablock and clear saved vectors
		return; //stop here
	}

	%pos = %player.getPosition(); //player position

	if(%player.AdvLedgeSafeVec[2] !$= "")
	{
		%min = (%player.AdvLedgeSafeVec[1] $= "" || %player.AdvLedgeSafeVec[0] $= "") ? 1.42 : 1.8; //if (second 'SafeVec' DNE, or first 'SafeVec' DNE) - use 1.3, else - use 1.8 //This is a fix for Corners, where both SafeVec's exist.
	//	%max = (%player.AdvLedgeSafeVec[1] $= "" || %player.AdvLedgeSafeVec[0] $= "") ? 1 : 0.25; //if (second 'SafeVec' DNE, or first 'SafeVec' DNE) - use 1.3, else - use 1.8 //This is a fix for Corners, where both SafeVec's exist.
		%syncVec = MoveInputCamSync(%player.client,"vector",%fvec); //get Input vector
	//	if(%syncVec !$= "" && (%dist = vectorDist(%syncVec,%player.AdvLedgeSafeVec[2])) < %max) //if 'syncVec' exists, and the distance between the 'syncVec' and cumulative 'SafeVec' is greater than %min.
		if(%syncVec !$= "" && (%dist = vectorDist(%syncVec,%player.AdvLedgeSafeVec[2])) >= %min) //if 'syncVec' exists, and the distance between the 'syncVec' and cumulative 'SafeVec' is greater than %min.
		{
			AdventureLedgeSafetyFinish(%player);
			return; //stop here
		}
	}

	%xVel = getWord(%vel,0) SPC "0 0"; //new vector made from isolated x component of player vel
	%xvNrm = vectorNormalize(%xVel); //normalized x component of player vel
	%yVel = "0" SPC getWord(%vel,1) SPC "0"; //vector made from isolated y component of player vel
	%yvNrm = vectorNormalize(%yVel); //normalized y component of player vel

	%offset = 0.8; //Ground check horizontal offset
	%xPos = vectorAdd(%pos,vectorScale(%xvNrm,%offset)); //x offset'd position (for checking for last terra firma)
	%yPos = vectorAdd(%pos,vectorScale(%yvNrm,%offset)); //y offset'd position (for checking for last terra firma)

	%xGroundD = isObject(AdventureBoxCheck(%player,%low,%height,%xPos,"","Ledge Safety","0 1 0")); //x adjusted ground check
	%yGroundD = isObject(AdventureBoxCheck(%player,%low,%height,%yPos,"","Ledge Safety","0 1 1")); //y adjusted ground check

	if((!%xGroundD || !%yGroundD) && %player.AdvLedgeSafeVec[2] $= "")
	{
	//	talk("Safety: x Ground D =" SPC %xGroundD);
	//	talk("Safety: y Ground D =" SPC %yGroundD);
	
		%player.AdvLedgeSafeVec[0] = !%xGroundD ? vectorScale(%xvNrm,1) : "";
		%player.AdvLedgeSafeVec[1] = !%yGroundD ? vectorScale(%yvNrm,1) : "";
		%player.AdvLedgeSafeVec[2] = vectorNormalize(vectorAdd(%player.AdvLedgeSafeVec[0],%player.AdvLedgeSafeVec[1]));

		%movVel = vectorScale(%player.AdvLedgeSafeVec[2],-1);
	//	%player.setVelocity(getWords(%movVel,0,1) SPC 0);
		%player.changedatablock(PlayerAdventureNoMove);
	//	%player.playthread(0,plant);
	}
}
//Restore Datablock and Clear Vectors
function AdventureLedgeSafetyFinish(%player,%onJump)
{
	talk("Safety Finish: Ledge Vector =" SPC %player.AdvLedgeSafeVec[2]);
	for(%n=0;%n<3;%n++)
		%player.AdvLedgeSafeVec[%n] = ""; //clear both saved 'LedgeSafeVec' values
	if(%onJump && $Pref::Adventure::LedgeSafety)
	{
		%fvec = %player.getForwardVector(); //player forward vector
		%syncVec = MoveInputCamSync(%player.client,"vector",%fvec); //get Input vector
		%player.setVelocity(getWords(vectorScale(%syncVec,PlayerAdventure.maxForwardSpeed+2),0,1) SPC 0); //Simulate the running jump velocity
		%player.setActionThread(Jump);
		serverPlay3d(jumpsound,%player.getPosition());
	//	%player.playthread(0,ShiftUp);
	}
	%player.changedatablock(PlayerAdventure); //change to main datablock
}