//SFX
datablock AudioProfile(JumpDoubleSound)
{
   filename    = "./JumpSecond.wav";
   description = AudioClose3d;
   preload = true;
};
datablock AudioProfile(JumpHighSound)
{
   filename    = "./JumpHigh.wav";
   description = AudioClose3d;
   preload = true;
};


//Fantasy Tool
datablock StaticShapeData(FantasyEmptyData)
{
	category = "Static Shapes";
	shapeFile = "./cube_col.dts";
};


//Playertypes
datablock PlayerData(PlayerFantasy : PlayerStandardArmor)
{
	canJet = false;
	canFantasy = true;

//	jumpForce = 1080;
	cameraMaxDist = 6;

	uiName = "Fantasy Player";
	showEnergyBar = false;
	rechargeRate = 0;
};

datablock PlayerData(PlayerFantasyNoJump : PlayerFantasy)
{
	canNoJump = true;
	jumpForce = 0;
	uiName = "";
};
datablock PlayerData(PlayerFantasyFall : PlayerFantasyNoJump)
{
	airControl = 0;
};

datablock PlayerData(PlayerFantasyHP : PlayerStandardArmor)
{
	mass = 1;
	boundingBox = "0.1 0.1 0.1";
	uiName = "";
};


//Silent Wand FX
datablock ExplosionData(SilentWandExplosion : WandExplosion)
{
	soundProfile = "";
	shakeCamera = false;
};
datablock ProjectileData(SilentWandProjectile)
{
	explosion = SilentWandExplosion;
};

//Wall Jump Emitter
datablock ParticleData(FantasyJumpExplosionParticle)
{
	dragCoefficient      = 8;
	gravityCoefficient   = 2.5;
	inheritedVelFactor   = 0.2;
	lifetimeMS           = 300;
	lifetimeVarianceMS   = 100;
	textureName          = "base/data/particles/chunk";
	spinSpeed			= 10.0;
	spinRandomMin		= -50.0;
	spinRandomMax		= 50.0;

	colors[0]     = "0.8 0.8 0.8 0.1";
	colors[1]     = "0.8 0.8 0.8 0";
	sizes[0]      = 0.6;
	sizes[1]      = 0.8;
	times[0]	  = 0;
	times[1]	  = 1;

	useInvAlpha = true;
};
datablock ParticleEmitterData(FantasyJumpExplosionEmitter)
{
   ejectionPeriodMS = 1;
   ejectionVelocity = 6;
   velocityVariance = 1;
   thetaMin         = 89;
   thetaMax         = 90;
   phiVariance      = 360;
   particles = "FantasyJumpExplosionParticle";
};

//High Jump FX
datablock ExplosionData(FantasyHighJumpExplosion)
{
   lifeTimeMS = 150;

   emitter[0] = FantasyJumpExplosionEmitter;
   emitter[1] = PushBroomExplosionEmitter;

   particleDensity = 10;
   particleRadius = 0.2;

   faceViewer     = true;
   explosionScale = "1 1 1";
   
   shakeCamera = false;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1 1 1";
   camShakeDuration = 0.5;
   camShakeRadius = 5.0;
};
datablock ProjectileData(FantasyHighJumpProjectile)
{
   explosion = FantasyHighJumpExplosion;
};


//Damage System
package FantasyVignette
{
	function Armor::onDamage(%data,%obj,%damage)
	{
		Parent::onDamage(%data,%obj,%damage);
		if(%data.canFantasy)
		{
			%dmg = %obj.getDamageLevel();
			%max = %data.maxDamage;
			%ratio = 100/%max;
			%scale = %dmg/%ratio;
			%color = %scale*0.01;
			if(%scale == 100 || %scale == 0)
				commandToClient(%obj.client,'SetVignette',$EnvGuiServer::VignetteMultiply,$EnvGuiServer::VignetteColor);
			else
				commandToClient(%obj.client,'SetVignette',$EnvGuiServer::VignetteMultiply, 1 SPC 1-%color SPC 1-%color SPC %color);
			if(%damage > 0)
			{
				%obj.DmgDelayed = true;
				cancel(%obj.DmgDelayEnd);
				%obj.DmgDelayEnd = schedule($Pref::Fantasy::HealthRegenDelay,0,eval,%obj@".DmgDelayed = false;");
			}
			//3D Health Display
			%hp = 100-%scale;
		//	FantasyShowHealth(%obj,%hp,%color);
		}
	}
	function Armor::onAdd(%data,%obj)
	{
		Parent::onAdd(%data,%obj);
		commandToClient(%obj.client,'setVignette',$EnvGuiServer::VignetteMultiply,$EnvGuiServer::VignetteColor);
	}
	function gameConnection::onDeath(%killed,%killerPlayer,%killer,%a,%b)
	{
		if(isObject(%killed))
			commandToClient( %killed,'SetVignette',$EnvGuiServer::VignetteMultiply,$EnvGuiServer::VignetteColor);
		Parent::onDeath(%killed,%killerPlayer,%killer,%a,%b);
	}
	//HP Bot
	function Armor::onMount(%this,%obj,%col,%slot)
	{
		if(%obj.type !$= "FantasyHP")
			Parent::onMount(%this,%obj,%col,%slot);
	}
	function Armor::onUnmount(%this,%obj,%col,%slot)
	{
		if(%obj.type $= "FantasyHP")
			%obj.schedule(5,delete);
		Parent::onUnmount(%this,%obj,%col,%slot);
	}
};activatePackage(FantasyVignette);


package FantasyPlayer
{
	//on Add
	function Armor::onNewDatablock(%data,%obj)
	{
		%client = %obj.client;
		if(%client $= "")
			return;
		if(%data.getName() $= "PlayerFantasy" && !$Fantasy::SilentStarted)
			schedule(5,0,FantasyGlobalSilentLoop);
		Parent::onNewDatablock(%data,%obj);
		FantasyPaintToggle(%obj,%data);
		%obj.lastData = %data;
	}
	//on Death
	function Armor::onDisabled(%data,%obj,%enabled)
	{
		if(%data.canFantasy)
			FantasyStatusBar(%obj);
		Parent::onDisabled(%data,%obj,%enabled);
	}
	//Brick Gui Fix
	function WeaponImage::onMount(%this,%obj,%slot)
	{
		if(%this == 193) //Brick Image
			%obj.isBricking = true;
		Parent::onMount(%this,%obj,%slot);
	}
	function WeaponImage::onUnMount(%this,%obj,%slot)
	{
		if(%this == 193) //Brick Image
			%obj.isBricking = false;
		Parent::onUnMount(%this,%obj,%slot);
	}
	//Player Triggers
	function Armor::onTrigger(%data,%obj,%slot,%val)
	{
		Parent::onTrigger(%data,%obj,%slot,%val);
		if(!%data.canFantasy)
			return;
		if(%slot == 2)
		{
			if(%val)
			{
				if(%obj.isFantasyJet)
				{
					FantasyHighJump(%obj);
					if(!%obj.isFantasyJump2)
						FantasySecondJump(%obj);
				}
				else
					FantasyWallBounce(%obj);
			}
		}
		if(%slot == 3)
		{
			%obj.isFantasyCrouching = %val;
		//	if(!%val)
		//		cancel(%obj.FantasyAirDashSched);
		//	else
		//		%obj.FantasyAirDashSched = schedule(300,0,FantasyAirDash,%obj);
		}
		if(%slot == 4)
			%obj.isFantasyJet = %val;
	}
	//Activate
	function Player::activateStuff(%obj)
	{
		%data = %obj.getDatablock();
		if(%data.canFantasy)
			FantasyFlashDashCheck(%obj);
		else
			Parent::activateStuff(%obj);
	}
};activatePackage(FantasyPlayer);

//Disable Paint
function FantasyPaintToggle(%obj,%data)
{
	if(%data.canFantasy && %obj.cannotPaint $= "")
	{
		commandToClient(%obj.client,'SetPaintingDisabled',true);
		%obj.cannotPaint = "Fantasy";
	}
	else if(!%data.canFantasy && %obj.cannotPaint $= "Fantasy")
	{
		%client = %obj.client;
		%mini = %client.minigame;
		if(!isObject(%mini))
			%paint = true;
		else
			%paint = %client.minigame.enablePainting;
		commandToClient(%client,'SetPaintingDisabled',!%paint);
		%obj.cannotPaint = "";
	}
}


//Flash Step
function FantasyFlashDashCheck(%obj)
{
	if(!%obj.isFantasyFlashDashReady)
	{
		%obj.isFantasyFlashDashReady = true;
		schedule(400,0,eval,%obj@".isFantasyFlashDashReady = false;");
	}
	else if(%obj.isFantasyJet && !%obj.isFantasyFlashDash)
		FantasyFlashDashActual(%obj);
}
//Dash Actual
function FantasyFlashDashActual(%obj)
{
	%vec = %obj.getMuzzleVector(1);
	%hack = %obj.getHackPosition();

	%hit = FantasyBeamCheck(%obj,%hack,10,$TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType | $Typemasks::StaticShapeObjectType | $Typemasks::PlayerObjectType | $Typemasks::VehicleObjectType,%vec);
	if(isObject(%hit))
	{
		%loc = %obj.FantasyHitPos;
		%mov = vectorAdd(vectorScale(%vec,-3),%loc);
	}
	else
		%mov = vectorAdd(vectorScale(%vec,10),%hack);

	%space = !isObject(FantasyBoxCheck(%obj,2.6,%mov,"All"));
	if(%space)
		%pos = vectorAdd(%mov,"0 0" SPC -1.325);
	else
	{
		%obj.playthread(0,undo);
	//	messageClient(%obj.client,'MsgItemPickup','',"","");
		return;
	}

	%obj.playthread(0,plant);
	%obj.playthread(2,shiftdown);
	%obj.setVelocity(vectorScale(%vec,10));
	%obj.setTransform(%pos);
	FantasyFX(%obj,WandProjectile,%hack,%vec,1.5);
	FantasyFX(%obj,SilentWandProjectile,%mov,%vec,1);
	%obj.isFantasyFlashDash = true;
	schedule(1200,0,eval,%obj@".isFantasyFlashDash = false;");
}


//Fantasy Global Silent Loop
function FantasyGlobalSilentLoop()
{
	cancel($Fantasy::SilentSchedule);
	for(%i=0;%i<ClientGroup.getCount();%i++)
	{
		%client = ClientGroup.getObject(%i);
		if(!isObject(%obj = %client.player) || %obj.getState() $= "Dead")
			continue;
		%data = %obj.getDatablock();
		if(!%data.canFantasy)
			continue;
		FantasyHealthRegen(%obj);
		FantasyStatusBar(%obj);
		FantasyWallRun(%obj,%data);
		FantasyLandedCheck(%obj);
		%isFantasizer = true;
	}

	if(%isFantasizer)
	{
		$Fantasy::SilentSchedule = scheduleNoQuota(10,0,FantasyGlobalSilentLoop);
		$Fantasy::SilentStarted = true;
	}
	else
		$Fantasy::SilentStarted = false;
}


//Health Regeneration
function FantasyHealthRegen(%obj)
{
	if(!$Pref::Fantasy::HealthRegen)
		return;
	if(%obj.isFantasyRegened)
		return;
	cancel(%obj.AHRS);
	%obj.isFantasyRegened = true;
	%obj.AHRS = schedule(200,0,eval,%obj@".isFantasyRegened = false;");
	%dmg = %obj.getDamageLevel();
	if(%dmg > 0 && !%obj.DmgDelayed)
		%obj.setDamageLevel(%dmg-1);
}


//Health, Energy, etc. Display
function FantasyStatusBar(%obj)
{
	if($Pref::HataAmmo::Enabled)
		return;
	%data = %obj.getDatablock();
	%dmg = %obj.getDamageLevel();
	%max = %data.maxDamage;
	%ratio = 100/%max;
	%scale = %dmg/%ratio;
	%percent = 100-%scale;
	%gauge1 = FantasyGauge(%percent);
	%health = "<font:impact:36>\c6" @ %gauge1;
	if(!%obj.isBricking)
		%message = "<just:right><font:impact:22>" @ "<br><br><br><br>" @ %health SPC "<font:Impact:28>\c6+ ";
	bottomPrint(%obj.client,%message,1);
}
//Bar Gauge
function FantasyGauge(%val,%half,%quarter)
{
	%contA = "-----";
	%contB = "----------";
	if(%half)
	{
		%val = %val/2;
		%contB = "";
	}
	if(%quarter)
	{
		%val = %val/4;
		%contA = "";
		%contB = "";
	}
	if(%val < 5)
		%display = "\c7-----" @ %contA @ %contB;
	else if(%val < 10)
		%display = "-\c7----" @ %contA @ %contB;
	else if(%val < 15)
		%display = "--\c7---" @ %contA @ %contB;
	else if(%val < 20)
		%display = "---\c7--" @ %contA @ %contB;
	else if(%val < 25)
		%display = "----\c7-" @ %contA @ %contB;
	else if(%val < 30)
		%display = "-----\c7" @ %contA @ %contB;
	else if(%val < 35)
		%display = "------\c7----" @ %contB;
	else if(%val < 40)
		%display = "-------\c7---" @ %contB;
	else if(%val < 45)
		%display = "--------\c7--" @ %contB;
	else if(%val < 50)
		%display = "---------\c7-" @ %contB;
	else if(%val < 55)
		%display = "----------\c7" @ %contB;
	else if(%val < 60 && !%half)
		%display = "-----------\c7---------";
	else if(%val < 65 && !%half)
		%display = "------------\c7--------";
	else if(%val < 70 && !%half)
		%display = "-------------\c7-------";
	else if(%val < 75 && !%half)
		%display = "--------------\c7------";
	else if(%val < 80 && !%half)
		%display = "---------------\c7-----";
	else if(%val < 85 && !%half)
		%display = "----------------\c7----";
	else if(%val < 90 && !%half)
		%display = "-----------------\c7---";
	else if(%val < 95 && !%half)
		%display = "------------------\c7--";
	else if(%val < 100 && !%half)
		%display = "-------------------\c7-";
	else if(%val == 100 && !%half)
		%display = "--------------------";
	return %display;
}

//Air Dash
function FantasyAirDash(%obj)
{
	%floor = FantasyBoxCheck(%obj,1.2,"","Low"); //high enough
	if(isObject(%floor) || %obj.isFantasyJumpLagged || %obj.isFantasyDash || %obj.isPhysicsDelayed || !%obj.isFantasyJet)
		return;

	%vel = %obj.getVelocity();
	%vec = %obj.getMuzzleVector(1);
	%dot = vectorDot(%vel,%vec);

	%max = 10;
	%sub = %dot+%max;
	if(%sub > %max)
		%sub = %max;

	%fwd = vectorScale(%vec,%sub);
	%add = vectorAdd(%fwd,"0 0 5");
	%hack = %obj.getHackPosition();

	%obj.setVelocity(%add);
	%obj.playthread(0,plant);
	%obj.playthread(2,shiftdown);
	FantasyFX(%obj,WandProjectile,%hack,%vec,1);
	%obj.isFantasyDash = true;
}

//Super Jump
function FantasyHighJump(%obj)
{
	%floor = FantasyBoxCheck(%obj); //on ground
	if(!isObject(%floor))
		return;

	%pos = %obj.getPosition();
	%vel = %obj.getVelocity();
	%dv = getWords(%vel,0,1);

	%obj.playthread(0,jump);
//	%obj.playthread(0,shiftup);
	%obj.setVelocity(%dv SPC 18);
	FantasyFX(%obj,FantasyHighJumpProjectile,%pos,"0 0 1",1);
	serverplay3d(JumpHighSound,%pos);
	%obj.isFantasyJumpLagged = true;
	schedule(400,0,eval,%obj@".isFantasyJumpLagged = false;");
	%obj.changedatablock(PlayerFantasyNoJump);
	schedule(400,0,FantasyDatablockRestore,%obj);
}

//Double Jump
function FantasySecondJump(%obj)
{
	%floor = FantasyBoxCheck(%obj,1.2,"","Low"); //high enough
	if(isObject(%floor) || %obj.isFantasyJumpLagged)
		return;

	%pos = %obj.getPosition();
	%vel = %obj.getVelocity();
	%dv = getWords(%vel,0,1);
	%uv = getWord(%vel,2);
	if(%uv > -15)
	{
		%obj.playthread(0,jump);
	//	%obj.playthread(2,shiftup);
		%obj.setVelocity(%dv SPC 10+%uv/2);
	//	FantasyFX(%obj,FantasyHighJumpProjectile,%pos,"0 0 1",0.4);
		serverplay3d(JumpDoubleSound,%pos);
		%obj.changedatablock(PlayerFantasyNoJump);
		schedule(400,0,FantasyDatablockRestore,%obj);
		%obj.isFantasyJump2 = true;
		%obj.isFantasyJumpLagged = true;
		schedule(400,0,eval,%obj@".isFantasyJumpLagged = false;");
	}
}
function FantasyLandedCheck(%obj)
{
	%floor = FantasyBoxCheck(%obj); //on ground
	if(isObject(%floor))
	{
		%obj.isFantasyJump2 = false;
		%obj.isFantasyDash = false;
	}
}

//Wall Bounce
function FantasyWallBounce(%obj)
{
	%floor = FantasyBoxCheck(%obj,1.2,"","Low"); //high enough
	if(isObject(%floor) || %obj.isFantasyJumpLagged)
		return;
	%pos = %obj.getPosition();
	%ank = vectorAdd(%pos,"0 0 0.2");
	%vel = %obj.getVelocity();
	%upv = getWord(%vel,2);
	%vec[0] = %obj.getForwardVector();
	%vec[1] = vectorScale(%vec[0],-1);
	%vec[2] = getRelativeX(%obj,%vec[0]);
	%vec[3] = vectorScale(%vec[2],-1);
	for(%a=0;%a<4;%a++)
	{
		%dir = FantasyDirCheck(%obj,%vec[%a]);
		%isWall = isObject(FantasyBeamCheck(%obj,%ank,0.8,$TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,%dir));
		if(%isWall)
			break;
	}
	%off = vectorAdd(%pos,"0 0" SPC -1);
	if(%isWall && %upv > -10 && %upv != 0)
	{
		%xV = getWord(%vel,0);
		%yV = getWord(%vel,1);
		%dV = vectorDot(%vel,%dir);
		if(mAbs(%xV) == mAbs(%dV))
			%dF = 0 SPC %yV SPC 0;
		if(mAbs(%yV) == mAbs(%dV))
			%dF = %xV SPC "0 0";
		%yA = vectorScale(%dir,-4);
		%dA = vectorAdd(%dF,%yA);
		%zA = "0 0" SPC 10+%upv/2;
		%obj.setVelocity(vectorAdd(%zA,%dA));
		serverplay3d(jumpsound,%pos);
		%obj.playthread(0,jump);
		%obj.playthread(2,shiftdown);
		%mov = vectorScale(%dir,0.6);
		%add = vectorAdd(%pos,%mov);
		FantasyFX(%obj,FantasyHighJumpProjectile,%add,%dir,0.8);
		schedule(400,0,eval,%obj@".isFantasyJumpLagged = false;");
		%obj.isFantasyJumpLagged = true;
	}
}

//Wall Run
function FantasyWallRun(%obj,%data)
{
	%pos = %obj.getPosition();
	%off = vectorAdd(%pos,"0 0" SPC -1);
	%floor = FantasyBoxCheck(%obj,1,"","Low"); //high enough
	if(isObject(%floor) || %obj.isFantasyCrouching || %obj.isFantasyWallStep)
		return;
	%vel = %obj.getVelocity();
	%upv = getWord(%vel,2);
	if(%upv == 0)
		return;
	%ank = vectorAdd(%pos,"0 0 0.2");
	%vec[0] = %obj.getForwardVector();
	%vec[1] = vectorScale(%vec[0],-1);
	%vec[2] = getRelativeX(%obj,%vec[0]);
	%vec[3] = vectorScale(%vec[2],-1);
	for(%a=0;%a<4;%a++)
	{
		%dir = FantasyDirCheck(%obj,%vec[%a]);
		%isWall = isObject(FantasyBeamCheck(%obj,%ank,0.7,$TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,%dir));
		if(%isWall)
			break;
	}
	if(!%isWall)
		return;
	%bck = FantasyDirCheck(%obj,%vec[1]);
	%fwd = vectorScale(%bck,-1);
	if(%bck $= %dir)
		return;

	%x = %vec[2];
	if(%obj.isFantasyWallLeft)
	{
		%obj.isFantasyWallLeft = false;
		%x = %vec[3];
	}
	else
		%obj.isFantasyWallLeft = true;
	if(%fwd !$= %dir)
		%x = "0 0 0";
	%mov = vectorScale(%dir,0.6);
	%xMov = vectorScale(%x,0.3);
	%add = vectorAdd(%mov,%xMov);
	%addB = vectorAdd(%pos,%add);
	FantasyFX(%obj,FantasyHighJumpProjectile,%addB,%dir,0.5);
	schedule(100,0,eval,%obj@".isFantasyWallStep = false;");
	%obj.isFantasyWallStep = true;
}


//Fantasy 3D Health Display
function FantasyShowHealth(%obj,%hp,%color)
{
	%HPData = %obj.HPData;
	if(!isObject(%HPData))
	{
		%HPData = new AIPlayer()
		{
			dataBlock = PlayerFantasyHP;
			type = FantasyHP;
			scale = "0.6 0.6 0.3";
		};MissionCleanup.add(%HPData);
		%HPData.HideNode("ALL");
		%HPData.unHideNode("visor");
		%obj.schedule(5,mountObject,%HPData,6);
		%obj.HPData = %HPData;
	}

	%head = 1 SPC 1-%color SPC 1-%color SPC 1;
	%HPData.setNodeColor("visor",%head);

	cancel(%obj.HPBotKill);
	if(%hp == 100)
		%obj.HPBotKill = %HPData.schedule(500,delete);
}

//Fantasy Tool
function FantasyShape(%obj,%pos)
{
	%rot = getWords(%obj.getTransform(),3,6);
	%low = vectorAdd(%pos,"0 0 -0.11");
	%vec = %obj.getForwardVector();
	%fwd = vectorScale(%vec,0.4);
	%off = vectorAdd(%fwd,%low);
	%shape = %obj.client.FantasyTool;
	if(isObject(%shape))
	{
		%old = %shape.getPosition();
		%dst = vectorDist(%old,%off);
		if(%dst < 0.01)
			return;
	//	centerPrint(%obj.client,"<font:impact:30><color:FFF200><just:center>Last Shape   <color:FFFFFF>distance =" SPC %dst,1);
	}

	//Hang Platform Static Shape
	%shape = new StaticShape()
	{
		datablock = FantasyEmptyData;
		position = %off;
		client = %obj.client;
		scale = "0.4 0.2 0.1";
		type = ClimbHelper;
	};missionCleanup.add(%shape);
	%shape.setTransform(%off SPC %rot);
	FantasyShapeRemove(%obj.client);
	%obj.client.FantasyTool = %shape;
}
function FantasyShapeRemove(%client)
{
	if(isObject(%client.FantasyTool))
		%client.FantasyTool.delete();
}

//Restore Playertype
function FantasyDatablockRestore(%obj)
{
	if(%obj.getState() !$= "Dead" && %obj.getDatablock().canNoJump)
		%obj.changedatablock(PlayerFantasy);
}

//Special FX
function FantasyFX(%obj,%data,%pos,%vel,%scale)
{
	%p = new Projectile()
	{
		dataBlock = %data;
		initialPosition = %pos;
		initialVelocity = %vel;
		sourceObject = %obj;
		client = %obj.client;
		scale = %scale SPC %scale SPC %scale;
	};MissionCleanup.add(%p);
	%p.explode();
}

//Grid Snap
function FantasyDirCheck(%obj,%vec)
{
	%xa = "1 0 0";
	%ya = "0 1 0";
	%xb = "-1 0 0";
	%yb = "0 -1 0";
	%xad = vectorDist(%vec,%xa);
	%yad = vectorDist(%vec,%ya);
	%xbd = vectorDist(%vec,%xb);
	%ybd = vectorDist(%vec,%yb);
	%len = 0.75; //closest dir
	if(%yad < %len)
		%dir = %ya;
	if(%ybd < %len)
		%dir = %yb;
	if(%xad < %len)
		%dir = %xa;
	if(%xbd < %len)
		%dir = %xb;
	return %dir;
}
function FantasyDirFix(%obj,%dir)
{
	%yaw = mATan(getWord(%dir,1),getWord(%dir,0));
	%fix = "0 0 -1" SPC %yaw;
	%eul = axisToEuler(%fix);
	%eulA = vectorAdd(%eul,"0 0 -90");
	%grid = eulerToAxis(%eulA);
	return %grid;
}

//Box Search
function FantasyBoxCheck(%obj,%up,%start,%type)
{
	if(%up $= "")
		%up = 0.1;
	%pos = %obj.getPosition();
	%half = "0 0" SPC -%up/2;
	if(%start $= "")
		%start = %pos;
	if(%type $= "Low")
		%start = vectorAdd(%half,%start);
	if(%type $= "All")
		%masks = $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType | $Typemasks::StaticShapeObjectType | $Typemasks::PlayerObjectType | $Typemasks::VehicleObjectType;
	else
		%masks = $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType | $Typemasks::StaticShapeObjectType;
	initContainerBoxSearch(%start, "1.245 1.245" SPC %up, %masks);
	while(%col = containerSearchNext())
	{
		if(isObject(%col) && %col != %obj && %col.type !$= "ClimbHelper")
			return %col;
	}
}

//Beam Check
function FantasyBeamCheck(%obj,%pos,%len,%masks,%vec)
{
	%beam = vectorScale(%vec,%len);
	%end = vectorAdd(%pos,%beam);
	%ray = containerRayCast(%pos,%end,%masks,%obj);
	%col = firstWord(%ray);
	if(isObject(%col))
		%obj.FantasyHitPos = posFromRaycast(%ray);
	return %col; //hit true/false
}

//Euler to Axis & Axis to Euler
function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}
function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

//Relative Vector
function getRelativeX(%obj,%dir)
{
	return vectorCross(%dir,%obj.getUpVector());
}

//Rotate Vector
function vectorRotateAxis(%vec,%axis)
{
	//Epic function found online. Credits to Blocki <3. Id also like to thank Zeblote for finding this for me <3
	%u["x"] = getword(%axis,0);
	%u["y"] = getword(%axis,1);
	%u["z"] = getword(%axis,2);

	%angl = getword(%axis,3) * -1;
	%cos = mcos(%angl);
	%sin = msin(%angl);

	%a[1,1] = %cos + (%u["x"] * %u["x"] * (1 - %cos));
	%a[1,2] = (%u["x"] * %u["y"] * (1 - %cos)) - (%u["z"] * %sin);
	%a[1,3] = (%u["x"] * %u["z"] * (1 - %cos)) + (%u["y"] * %sin);

	%a[2,1] = (%u["y"] * %u["x"] * (1 - %cos)) + (%u["z"] * %sin);
	%a[2,2] = %cos + (%u["y"] * %u["y"] * (1 - %cos));
	%a[2,3] = (%u["y"] * %u["z"] * (1 - %cos)) - (%u["x"] * %sin);

	%a[3,1] = (%u["z"] * %u["x"] * (1 - %cos)) - (%u["y"] * %sin);
	%a[3,2] = (%u["z"] * %u["y"] * (1 - %cos)) + (%u["x"] * %sin);
	%a[3,3] = %cos + (%u["z"] * %u["z"] * (1 - %cos));

	%x = getWord(%vec, 0);
	%y = getWord(%vec, 1);
	%z = getWord(%vec, 2);

	%newx = (%a[1,1] * %x) + (%a[1,2] * %y) + (%a[1,3] * %z);
	%newy = (%a[2,1] * %x) + (%a[2,2] * %y) + (%a[2,3] * %z);
	%newz = (%a[3,1] * %x) + (%a[3,2] * %y) + (%a[3,3] * %z);

	%pos = %newx SPC %newy SPC %newz;
	return %pos;
}