//Playertype
datablock PlayerData(PlayerPhysicsJump : PlayerStandardArmor)
{
	canJet = false;
	canPhysicsJump = true;

	uiName = "Phys. Jump Player";
	showEnergyBar = false;
};


//Vehicle
datablock WheeledVehicleData(PhysicsRollVehicle)
{
	//Basic Settings
	category = "Vehicles";
	shapeFile = "./ball22.dts";
	emap = true;
	uiName = "";

	//Vehicle Settings
	doSimpleDismount = false;		//just unmount the player, dont look for a free space
	numMountPoints = 1;
	isPhysicsJump = true;
	maxDamage = 999999;
	destroyedLevel = 200;
	rideable = false;
	numWheels = 0;
	cameraRoll = false;

	//Advanced Steering
	maxSteeringAngle = 0.2;  // Maximum steering angle, should match animation
	steeringUseStrafeSteering = false;

	//Rigid Body
	mass = 200;
	massCenter = "0 0 1";    // Center of mass for rigid body
	massBox = "1 1 1";       // Size of box used for moment of inertia, if zero it defaults to object bounding box

	density = 1;
	drag = 0; //0.2; //was 0.8    // Drag coefficient
	bodyFriction = 0; //0.5; //was 0.21
	bodyRestitution = 0; //was 0.2

	minImpactSpeed = 0.1;        // Impacts over this invoke the script callback
	softImpactSpeed = 3;       // Play SoftImpact Sound
	hardImpactSpeed = 10;      // Play HardImpact Sound

	integration = 4;           // Physics integration: TickSec/Rate
	collisionTol = 0.3; //0.25        // Collision distance tolerance
	contactTol = 0.1;          // Contact velocity tolerance

	justcollided = false;
	isSled = true;	//if its a sled, the wing surfaces dont work unless its on the ground

	minRunOverSpeed    = 25;   //how fast you need to be going to run someone over (do damage)
	runOverDamageScale = 0;   //when you run over someone, speed * runoverdamagescale = damage amt
	runOverPushScale   = 0; //how hard a person you're running over gets pushed

	protectPassengersBurn   = false;  //protect passengers from the burning effect of explosions?
	protectPassengersRadius = false;  //protect passengers from radius damage (explosions) ?
	protectPassengersDirect = false; //protect passengers from direct damage (bullets) ?
};


package PhysicsJump
{
	//Camera Spy Fix
	function SimObject::onCameraEnterOrbit(%player,%camera)
	{
		if(%player.isPhysicsDelayed)
			%player.observerCount++;
		else
			Parent::onCameraEnterOrbit(%player,%camera);
	}
	function SimObject::onCameraLeaveOrbit(%player,%camera)
	{
		if(%player.isPhysicsDelayed)
			%player.observerCount--;
		else
			Parent::onCameraLeaveOrbit(%player,%camera);
	}
	//Vehicle Add
	function WheeledVehicleData::onAdd(%this,%vehicle)
	{
		%mov = vectorAdd(%vehicle.position,"0 0 0.2");
		if(%this.isPhysicsJump)
			%vehicle.setTransform(%mov SPC %vehicle.rotation);
		Parent::onAdd(%this,%vehicle);
	}
	//on Driver Leave Vehicle Deletion
	function WheeledVehicleData::onDriverLeave(%this,%vehicle,%player)
	{
		Parent::onDriverLeave(%this,%vehicle,%player);
		if(%this.isPhysicsJump)
			schedule(1,0,PhysicsJumpVehicleDelete,%vehicle);
	}
	//Item Pick up
	function WheeledVehicleData::onCollision(%this,%vehicle,%col,%a,%b,%c,%d,%e,%f)
	{
		%player = %vehicle.getMountedObject(0);
		%ret = Parent::onCollision(%this,%vehicle,%col,%a,%b,%c,%d,%e,%f);
		if(%this.isPhysicsJump && isObject(%player))
		{
			if(%col.canPickup)
			{
				for(%a=0;%a<%player.getDatablock().maxTools;%a++)
				{
					%tool = %player.tool[%a];
					%colT = %col.getDatablock();
					if(%colT == %tool)
						return %ret;
				}
				%player.pickup(%col);
			}
		}
		return %ret;
	}
	//onImpact
	function WheeledVehicleData::onImpact(%this,%vehicle,%col,%vec,%speed)
	{
		%player = %vehicle.getMountedObject(0); //vehicle's driver
		if(!isObject(%player) || !%this.isPhysicsJump) //if player DNE or vehicle is not Physics Jump Vehicle
			return; //stop here
	//	PhysSpeedCheck(%player,%vehicle,%vec,%speed); //Apply fall damage if applicable
	}
	//Projectile Stuff
	function ProjectileData::Damage(%this,%proj,%col,%fade,%pos,%normal)
	{
		if(%col.isPhysicsJump) //if vehicle is Physics Jump Vehicle
		{
			%player = %col.getMountedObject(0); //vehicle's driver
			if(%player.getType() & $TypeMasks::PlayerObjectType) //if driver is a player
				%player.damage(%proj.sourceObject,"",%this.directDamage,%this.damageType); //relay damage to player
		}
		else //otherwise
			Parent::Damage(%this,%proj,%col,%fade,%pos,%normal); //Parent default fcns
	}
	//Vehicle Mount/Unmount
	function Armor::onMount(%this,%player,%col,%slot)
	{
		if(%col.dataBlock.isPhysicsJump) //if vehicle is Physics Jump Vehicle
			%player.setControlObject(%col); //manually set player control of vehicle
		else //otherwise
			Parent::onMount(%this,%player,%col,%slot); //Parent default fcns
	}
	function Armor::onUnmount(%this,%player,%col,%slot)
	{
		if(%col.dataBlock.isPhysicsJump) //if vehicle is Physics Jump Vehicle
		{
			%tra = %col.getTransform(); //vehilce transform
			%player.setTransform(%tra); //move player to vehicle location
			schedule(1,0,eval,%player@".canDismount = true;"); //reenable player dismounting
		}
		Parent::onUnmount(%this,%player,%col,%slot);
	}
	//Camera Triggers
	function Observer::onTrigger(%data,%camera,%slot,%val)
	{
		%player = %camera.player;
		if(isObject(%player) && %player.hasPJData) //if player exists, and at least one Roll/Flip condition is met
		{
			if(%slot == 3 && %val || %slot == 2 && %val) //if ('on Crouch' and 'on Press'), or ('on Jump' and 'on Press')
				%player.isRollReady = true; //store hold/release (of Jump / Crouch key) to player object
			else if(%slot == 3 || %slot == 2) //otherwise, if (on Crouch, or on Jump)
				%player.isRollReady = false; //store hold/release (of Jump / Crouch key) to player object
		}
		Parent::onTrigger(%data,%camera,%slot,%val);
	}
	//Player Triggers
	function Armor::onTrigger(%data,%player,%slot,%val)
	{
		Parent::onTrigger(%data,%player,%slot,%val);
		%adv = $Pref::PhysicsJump::Adventure && %data.canAdventure; //Pref: Adventure-Int on, and in Adventure Player
		%king = $Pref::PhysicsJump::KH && %data.canKH; //Pref: KH-Int on, and in KH Player
		%mini = %player.client.minigame !$= "" || !$Pref::PhysicsJump::MinigameOnly; //minigame exists, or Pref: Minigame-Only off
		%main = $Pref::PhysicsJump::Roll || $Pref::PhysicsJump::Flip || %data.canPhysicsJump; //main prefs, or Physics Jump Player
		%player.hasPJData = %main || %adv || %king || %mini; //Save whether any conditions are met
		if((%slot == 3 || %slot == 2) && %val && %player.hasPJData) //if (Crouching or Jumping), and pressed, and at least one Roll/Flip condition is met
		{
			if((%data.canPhysicsJump || %player.isPhysicsRollReady) && !%player.isPhysicsDelayed) //if (Physics Jump Playertype, or Double Tapped Shift), and did not just Roll/Flip
				PhysicsJumpRoll(%player,%data); //start Roll/Flip
			else //otherwise
			{
				%player.isPhysicsRollReady = true; //double tap winodw active
				schedule(300,0,eval,%player@".isPhysicsRollReady = false;"); //schedule end of Double Tap time window
			}
		}
	}
};activatePackage(PhysicsJump);

//on Impact Speed Check
function PhysSpeedCheck(%player,%veh,%vec,%speed)
{
	%pos = %player.getPosition();
	%vel = vectorScale(%vec,-1);
//	echo(%player.getShapeName() SPC "  speed =" SPC %speed SPC "  vel =" SPC %vel SPC "  vec =" SPC %vec);
	if(%speed < %player.getDatablock().minImpactSpeed)
		return;
	%nrm = vectorNormalize(%vel);
	%bck = vectorScale(%nrm,-0.3);
	%add = vectorAdd(%pos,%bck);
	stopPhysicsRoll(%player,true);
	%player.setTransform(%add);
	%player.setVelocity(%vel);
}

//Start Roll/Flip
function PhysicsJumpRoll(%player,%data)
{
	if(!isObject(%player) || %player.getState() $= "Dead")
		return;
	%pos = %player.getPosition();
	%isFloor = PhysBoxCheck(%player);
	%hack = %player.getHackPosition();
	%vel = %player.getVelocity();
	%vec = vectorNormalize(%vel);
	%xyv = getWords(%vel,0,1) SPC 0;
	%xyvL = vectorLen(%xyv);
	%z = getWord(%vel,2);
//	echo("xy vel =" SPC %xyv SPC "  xy vel length =" SPC %xyvL);
	%flip = !%isFloor && $Pref::PhysicsJump::Flip;
	%roll = %isFloor && $Pref::PhysicsJump::Roll;
	%special = %data.canPhysicsJump || ($Pref::PhysicsJump::Adventure && %data.canAdventure) || ($Pref::PhysicsJump::KH && %data.canKH && !%player.hasKG);
	if(getWord(%hack,2)-getWord(%pos,2) < 0.7) //Make sure player is not crouched
		return; //otherwise
	if((%roll || %flip || %special) && (%xyvL > 0.5 && %z > -15))
		startPhysicsRolling(%player,%flip,%vel,%z);
}
function startPhysicsRolling(%player,%flip,%vel,%z)
{
	if(isObject(%player.PJCar)) //if Physics Jump Vehicle exists already
		return; //stop here

	//Properties
	%player.canDismount = false;
	%player.isPhysicsDelayed = true;

	//General
	%pos = %player.getPosition();
	%rot = rotationFromVector(%vel);

	//Spawn the Physics Vehicle
	%vehicle = new WheeledVehicle() 
	{
		dataBlock = PhysicsRollVehicle;
		client = %player.client;
		position = %pos;
		rotation = %rot;
		scale = "0.7 0.8 0.8"; //set its scale appropriately
	};MissionCleanup.add(%vehicle);
	%player.PJCar = %vehicle; //store vehicle to player object

	//Animate
	%player.playthread(0,sit);
	%player.playthread(2,armreadyboth);

	//Camera
	%data = %player.getDatablock();
	%dist = %data.cameraMaxDist;
	%dist = %dist < 4 ? 4 : %dist; //if dist less than 4 - make it 4, otherwise - keep it
	%tra = %player.getTransform(); //Player direction
	%camera = %player.client.camera;
	%camera.mode = "PhysicsJump";
	%camera.player = %player;
	%camera.setOrbitMode(%player,%pos,0,%dist,%dist,true);
	%camera.setTransform(%tra);
	%player.client.setControlObject(%camera);

	//Roll Speed and Time
	%time = 530; //delay before flip/roll automatically ends
	%pitch = -2; //rotation speed
	%buff = 3; //set speed boost //was 1.25
	%pitch = %flip ? %pitch : %pitch*1.2; //if flip - keep rotiatonal force, else boost it (if roll)
//	%time = %flip ? %time*2 : %time; //if flip - set higher timeout (for a double flip), else keep timeout

	//Schedules
	cancel(%player.PRCheck); //cancel any previous schedules
	%player.PRCheck = schedule(%time,0,stopPhysicsRollCheck,%player);
	schedule(1,0,PhysVehicleMount,%player,%vehicle,%vel,%pitch,%tra,%buff);
}

//Mount and Apply Velocity
function PhysVehicleMount(%player,%vehicle,%vel,%pitch,%traA,%buff)
{
	//Rotation
	%traB = %vehicle.getTransform();
	%rotA = getWords(%traA,3,6); //player rotation
	%eulA = axisToEuler(%rotA); //euler rot of player
	%rotB = getWords(%traB,3,6); //vehicle rotation
	%eulB = axisToEuler(%rotB); //euler rot of vehicle

	//Velocity
	%nrm = vectorNormalize(%vel);
	%vec = %player.getForwardVector();
	%nxy = getWords(%nrm,0,1) SPC 0;
	%speed = vectorScale(%nxy,%buff); //speed buff multiplier
	%add = vectorAdd(%speed,%vel); //add buff to current velocity
	%vehicle.mountObject(%player,0); //mount player to vehicle
	%vehicle.setVelocity(%add); //set initial vehicle velocity

	//Magic
	%rvc = findRelativeX(%vec); //rightward vector
	%dot = vectorDot(%rvc,%vel); //rightward velocity
	%yaw = getWord(%eulB,2) - getWord(%eulA,2); //find relative yaw
	%yaw = %dot > 0.5 || %dot < -0.5 ? getWord(%eulA,2) - getWord(%eulB,2) : %yaw; //if side speed is over 0.5 - invert relative yaw, else keep it
	%fix = "0 0" SPC %yaw; //euler yaw

	//Do Rotation
	%pos = %vehicle.getPosition();
	%player.setTransform(%pos SPC eulerToAxis(%fix)); //convert back to axis and rotate (inside vehicle)
	PhysVehicleRotate(%player,%vehicle,%pitch); //set angular velocity
}
function PhysVehicleRotate(%player,%vehicle,%pitch)
{
	%vec = %pitch SPC "0 0";
	%tra = %vehicle.getTransform();
	%rot = getWords(%tra,3,6);
	%fix = vectorRotateAxis(%vec,%rot);
	%vehicle.setAngularVelocity(%fix);
}

//Check for Roll Continue
function stopPhysicsRollCheck(%player)
{
	%pos = %player.getPosition();
	%isFloor = PhysBoxCheck(%player,4);
	if(%player.isRollReady && !%isFloor)
		schedule(50,0,stopPhysicsRollCheck,%player);
	else
		stopPhysicsRoll(%player);
}

//Stop Rolling/Flipping
function stopPhysicsRoll(%player,%force)
{
	%vehicle = %player.PJCar; //get Player's Physics Jump vehicle
	if(!isObject(%vehicle)) //if vehicle DNE
		return; //stop here
	cancel(%player.PRCheck); //cancel any previous schedules
	%pos = %vehicle.getPosition();
	%player.dismount();
	PhysicsJumpVehicleDelete(%vehicle);
	%camera = %player.client.camera;
	%off = vectorAdd(%pos,"0 0" SPC 1.325);
	%block = PhysBoxCheck(%player,2.5,%off);
//	echo(%player.getShapeName() SPC "   dismount pos =" SPC %pos SPC "  is blocked =" SPC %block);
	if(%block || %force)
		%pos = %player.getPosition(); //if blocked or forced, don't adjust position
	if(%player.getState() !$= "Dead")
	{
		%tra = %camera.getTransform();
		%rot = getWords(%tra,3,6);
		%player.setTransform(%pos SPC %rot);
		%player.playthread(2,shiftdown);
		%player.playthread(0,plant);
		schedule(300,0,eval,%player@".isPhysicsDelayed = false;");
		schedule(50,0,PhysCamRestore,%player,%camera);
	}
}
//Vehicle Delete
function PhysicsJumpVehicleDelete(%vehicle)
{
	if(isObject(%vehicle)) //if vehicle exists
		%vehicle.delete(); //delete it
}


//Return to Player Camera
function PhysCamRestore(%player,%camera)
{
	if(%camera.mode $= "PhysicsJump")
	{
		%player.client.setControlObject(%player);
		%camera.mode = "Observer";
		%camera.setFlyMode();
	}
}

//Box Check
function PhysBoxCheck(%player,%z,%start,%xy)
{
	if(%start $= "")
		%start = %player.getPosition();
	if(%z $= "")
		%z = 0.1;
	if(%xy $= "")
		%xy = 1.235;
	%masks = $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType | $Typemasks::StaticShapeObjectType | $Typemasks::PlayerObjectType;
	initContainerBoxSearch(%start, %xy SPC %xy SPC %z, %masks);
	while(%col = containerSearchNext())
	{
		if(isObject(%col) && %col != %player)
		{
			%cPos = %col.getPosition();
			%cXY = getWords(%cPos,0,1);
			%sXY = getWords(%start,0,1);
			%cZ = getword(%cPos,2);
			%sZ = getWord(%start,2);
			%sub = vectorSub(%cXY SPC 0,%sXY SPC 0);
			%nrm = vectorNormalize(%sub);
			%scl = vectorScale(%nrm,%xy/2);
			%add = vectorAdd(%start,%scl);
			%nPos = vectorAdd(%add,"0 0" SPC %z);
			%end = vectorAdd(%nPos,"0 0" SPC -%z*2);
			%ray = containerRayCast(%nPos,%end,%masks,%player);
			%hit = firstWord(%ray);
		//	centerPrint(%player.client,"<font:impact:30><color:FFF200><just:center>Physics Box<color:CCCCCC><br>" @ "col =" SPC %col SPC "  hit =" SPC %hit SPC "  start =" SPC %start @ "<br>sub =" SPC %sub @ "<br><color:FFFFFF>col pos =" SPC %cPos SPC "  new pos =" SPC %nPos,5);
			if(isObject(%hit) && %col == %hit)
				return %col;
		}
	}
}

//Relative Vector
function findRelativeX(%vec)
{
	%dir = getWords(%vec,0,1) SPC 0; //Make horizontal
	%nrm = vectorNormalize(%dir); //normalize
	return vectorCross(%dir,"0 0 1"); //rightward vector
}

//Rotation from Vector
function RotationFromVector(%vec)
{
	%yaw = mATan(getWord(%vec,1),getWord(%vec,0));
	%fix = "0 0 -1" SPC %yaw;
	%eul = axisToEuler(%fix);
	%eulA = vectorAdd(%eul,"0 0 -90");
	%rot = eulerToAxis(%eulA);
	return %rot;
}
//Rotate a Vector
function vectorRotateAxis(%vec,%axis)
{
	return matrixMulVector("0 0 0 " @ %axis,%vec);
}

//Euler to Axis & Axis to Euler
function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}
function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}