//Playertype
datablock PlayerData(PlayerSquid : PlayerStandardArmor)
{
	canJet = false;
	canSquid = true;
	jumpForce = 1280;

	uiName = "Squid Player";
	showEnergyBar = false;
};
datablock PlayerData(PlayerSquidSwim : PlayerSquid)
{
	maxForwardCrouchSpeed = 20;
	maxBackwardCrouchSpeed = 20;
	maxSideCrouchSpeed = 20;
	uiName = "";
};

package SquidPlayer
{
	function Armor::onNewDatablock(%data,%obj)
	{
		Parent::onNewDatablock(%data,%obj);
		if(%data.getName() $= "PlayerSquid")
		{
			if(!$Squid::SilentStarted)
				schedule(10,0,SquidGlobalSilentLoop);
		}
	}
	function Armor::onTrigger(%data,%obj,%slot,%val)
	{
		Parent::onTrigger(%data,%obj,%slot,%val);
		if(!%data.canSquid)
			return;
		if(%slot == 3)
			%obj.isSquidCrouching = %val;
		if(%slot == 4)
			%obj.isSquidJetting = %val;
	}
};activatePackage(SquidPlayer);

//Squid Global Silent Loop
function SquidGlobalSilentLoop()
{
	cancel($Squid::SilentSchedule);
	for(%i=0;%i<ClientGroup.getCount();%i++)
	{
		%client = ClientGroup.getObject(%i);
		if(!isObject(%obj = %client.player) || %obj.getState() $= "Dead")
			continue;
		%data = %obj.getDatablock();
		if(!%data.canSquid)
			continue;
		SquidSwim(%obj);
		%isSquider = true;
	}

	if(%isSquider)
	{
		$Squid::SilentSchedule = scheduleNoQuota(10,0,SquidGlobalSilentLoop);
		$Squid::SilentStarted = true;
	}
	else
		$Squid::SilentStarted = false;
}

//Swim on correctly colored bricks
function SquidSwim(%obj)
{
	%pos = %obj.getPosition(); //player position
	%ground = SquidBoxCheck(%obj,0.1); //floor
	%wep = %obj.getMountedImage(0); //weapon
	if(isObject(%ground) && isObject(%wep) && %wep.isPaintballGun)
	{
		%this = %wep.projectile; //weapon's projectile
		%colorA = %ground.getColorID(); //ground color
		%rgba = SquidBrickColorCheck(%this); //check paintgun color
		%colorB = getClosestPaintColor(%rgba); //get correct color
		%on = %colorA == %colorB; //if colors match
	}
	SquidSwimToggle(%obj,%on,%rgba); //Start/Stop swimming
}
//Toggle Swimming
function SquidSwimToggle(%obj,%on,%rgba)
{
	%data = %obj.getDatablock();
	if(%on && %obj.isSquidJetting && %obj.isSquidCrouching && %data.getName() !$= "PlayerSquidSwim")
	{
		%obj.changedatablock(PlayerSquidSwim); //psuedo swim
		%obj.setNodeColor("ALL",%rgba); //color player to paint
	}
	else if((!%on || !%obj.isSquidJetting || !%obj.isSquidCrouching) && %data.getName() $= "PlayerSquidSwim")
	{
		%obj.changedatablock(PlayerSquid); //restore playertype
		%obj.client.applyBodyColors(); //Fix body colors
	}
}

//Check brick color
function SquidBrickColorCheck(%this)
{
	if(%this.getName() $= "PaintballYellowProjectile")
		%rgb = "1 1 0";
	if(%this.getName() $= "PaintballOrangeProjectile")
		%rgb = "1 0.5 0";
	if(%this.getName() $= "PaintballGreenProjectile")
		%rgb = "0 1 0";
	if(%this.getName() $= "PaintballBlueProjectile")
		%rgb = "0 0 1";
	if(%this.getName() $= "PaintballRedProjectile")
		%rgb = "1 0 0";
	return %rgb SPC 1; //solid rgba color
}

//Box Check
function SquidBoxCheck(%obj,%z,%pos,%xy)
{
	if(%pos $= "")
		%pos = %obj.getPosition();
	if(%xy $= "")
		%xy = 1.25;
	initContainerBoxSearch(%pos, %xy SPC %xy SPC %z, $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType);
	%col = containerSearchNext();
	return %col;
}

//Credit to Space Guy for this
function getClosestPaintColor(%rgba)
{
	%prevdist = 100000;
	%colorMatch = 0;
	for(%i = 0;%i < 64;%i++)
	{
		%color = getColorIDTable(%i);
		if(vectorDist(%rgba,getWords(%color,0,2)) < %prevdist && getWord(%rgba,3) - getWord(%color,3) < 0.3 && getWord(%rgba,3) - getWord(%color,3) > -0.3)
		{
			%prevdist = vectorDist(%rgba,%color);
			%colormatch = %i;
		}
	}
	return %colormatch;
}