/////////////////////
//Movement Cam Sync//
/////////////////////

//Movement & Camera Sync
function MoveInputCamSync(%client,%syncDir,%lookVec,%ledgeDir,%range)
{
	//Use Global WASD Settings
	if(!$Pref::Adventure::FluidTraverse) //if Pref: Fluid-Traverse off
	{
		//use flat keyboard inputs
		if(%syncDir $= "right" && %client.KeyMoveRight) //if Sync Direction = "right", and client is holding 'moveRight' key
			return true; //stop and return true
		if(%syncDir $= "left" && %client.KeyMoveLeft) //if Sync Direction = "left", and client is pressing 'moveLeft' key
			return true; //..
		if(%syncDir $= "forward" && %client.KeyMoveForward) //if Sync Direction = "forward", and client is pressing 'moveForward' key
			return true; //..
		if(%syncDir $= "backward" && %client.KeyMoveBackward) //if Sync Direction = "backward", and client is pressing 'moveBackward' key
			return true; //..
		return false; //stop here (no matter what), and return false
	}

	//Look Vectors
	%fvec = vectorNormalize(getWords(%lookVec,0,1) SPC 0); //horizontal look vector (forward)
	%rvec = findRelativeX(%fvec); //rightward look vector
	%lvec = vectorScale(%rvec,-1); //leftward look vector
	%bvec = vectorScale(%fvec,-1); //backward look vector

	//Build Input-Vector (relative to Look vector)
	%inVec = "0 0 0"; //set input vector to zero by default
	if(%client.KeyMoveRight) //holding D
	{
		%keys++; //count the keys pressed
		%inVec = %rvec; //use rightward vector
	}
	if(%client.KeyMoveLeft) //holding A
	{
		%keys++; //+1
		%inVec = vectorAdd(%inVec,%lvec); //combine the vectors
	}
	if(%client.KeyMoveForward) //holding W
	{
		%keys++; //+1
		%inVec = vectorAdd(%inVec,%fvec); //..
	}
	if(%client.KeyMoveBackward) //holding S
	{
		%keys++; //+1
		%inVec = vectorAdd(%inVec,%bvec); //..
	}
	//Finished Move-Input Vector
	%nrmVec = vectorNormalize(%inVec); //normalize combined input vector

	//for returning Move-Input Vector
	if(%syncDir $= "vector") //if %syncDir arg. = "vector"
		return %nrmVec; //return the normalized movement input vector

	//Key Overlap checks	
	if(%keys > 2 || vectorLen(%inVec) == 0) //if pressing more than 2 keys, or pressing opposite keys (e.g. W + S)
		return false; //stop here, return bool 0

	//Ledge-facing Vectors
	%vec0 = findRelativeX(%ledgeDir); //player rvec
	%vec1 = %ledgeDir; 				  //player fvec

	//Compare Move-Input & Ledge-facing Vectors
	%dotx = vectorDot(%nrmVec,%vec0); //relative x vec
	%doty = vectorDot(%nrmVec,%vec1); //relative y vec

	//Return Comparison Results
	if(%syncDir $= "right" && %dotx > %range)
		return true; //stop and return true
	if(%syncDir $= "left" && %dotx < -%range)
		return true; //..
	if(%syncDir $= "forward" && %doty > %range)
		return true; //..
	if(%syncDir $= "backward" && %doty < -%range)
		return true; //..

	//If no matches made
	return false; //stop here, return bool 0
}


///////////////////////
//Single-Ledge Search//
///////////////////////

//Ledge Search
function AdventureLedgeSearch(%player,%posA,%len,%vec,%getPos)
{
	//Arguments
	%len = %len $= "" ? 1 : %len; //if len undefined - use global default var., else - use len arg.
	%vec = %vec $= "" ? %player.getForwardVector() : %vec; //if vec undefined - use player forward vector, else - use vec arg.

	//Raycast Starting-points
	%start0 = vectorAdd(%posA,"0 0" SPC $Adventure::PosA_offset); //hackPosition add (0.33 by default)
	%start1 = vectorAdd(%posA,"0 0" SPC $Adventure::PosB_offset); //eyePoint sub (0.4 by default)

	//Raycast Ending-points
	%beam = vectorScale(%vec,%len);
	%end0 = vectorAdd(%start0,%beam);
	%end1 = vectorAdd(%start1,%beam);

	//Do Raycast 0 (and get returned result)
	%ray0 = containerRayCast(%start0,%end0,$TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,%player); //Fire raycast
	%col0 = firstWord(%ray0); //Ledge

	if(isObject(%col0)) //if brick above 'Ledge Search' brick exists 
	{
		%ray1 = containerRayCast(%start1,%end1,$TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,%player);
		%col1 = firstWord(%ray1); //Space above ledge
	}

	%isLedge = isObject(%col0) && (!isObject(%col1) || $Pref::Adventure::BorderClimb && %col1 != %col0); //if col0 exists, and (col1 does not exist, or Pref: Border-Climbing and col1 is not col0)

	if(%isLedge) //if 'is Ledge' = true (Ledge found)
		%player.hangCol = isObject(%col0) ? %col0 : %player.hangCol; //if Ledge found - store the brick ID to player as "hangCol", else - keep previous 'hang Col'
	
	//Debugging Features
	if(%isLedge) //if ledge found
		AdventureLedgeIndicate(%player,%start0,%vec,%col0); //Place a ghost brick in front of the found ledge

	%max = %isLedge ? 2 : 1; //if 'isLedge' = true - set max to 2, else - set max to 1 (this way it does not show the second Raycast Debugger if a ledge was not found)
	for(%a=0;%a<%max;%a++) //loop from a = 0, to a = 1 (both col's and ray's)
	{
		if(isObject(%col[%a])) //if ray hit something
			%hit = posFromRaycast(%ray[%a]); //use hit position
		else //otherwise
			%hit = %end[%a]; //use raycast endpoint
		RaycastDebugUpdate(%player,%start[%a],%hit,"Ledge Search \[Case " @ %a @ "\]"); //Show Raycast Debugging shape
	}
	
	if(%getPos) //if input arg. 'get Position' = true, and ledge found
		%isLedge = %isLedge SPC posFromRaycast(%ray0); //add raycast hit position to the return string

	return %isLedge; //return 'ledge found / not found' as a true/false bool
}


//////////////////////
//Multi-Ledge Search//
//////////////////////

//Multi-Ledge Check
function AdventureMultiLedgeCheck(%player,%posA,%dir,%center,%lim,%len)
{
	%zPosA = getWord(%posA,2); //isolate hack Z val
	%round = mCeil(%zPosA*10)/10; //multiply by 10, round up, then divide by 10 (to round up to the nearest tenth, e.g. 1.215773 -> 1.3)
	%bPosA = getWords(%posA,0,1) SPC %round; //create vertical brick grid-adjusted PosA (hack position)
//	centerPrint(%player.client,"<font:impact:30><just:center>\c3Multi<br><color:CCCCCC>" @ %posA @ "<br><color:FFFFFF>" @ %nPosA,1);

	%lim = %lim $= "" ? 2 : %lim; //if lim undefined - use 2, else - use lim input arg.
	%half = mCeil(%lim/2); //rounds up half the lim
	for(%h=0;%h<%lim;%h++) //loop from h = 0, to h = %lim-1
	{
		%add = -%h*0.1; //relative offset for ledge search (changes with loop)
		%sub = %half*0.1; //offset the search upward by half the limit (e.g. if limit is 10, start from +0.5 and end at -0.5)
		%sub = %center ? %sub : 0.1; //if 'center' input arg. is true - keep z value, else - use 0
		%off = vectorAdd("0 0" SPC %add,"0 0" SPC %sub); //combine the offsets
		%mPosA = vectorAdd(%bPosA,%off); //offset'd the hack position
		%isLedge = AdventureLedgeSearch(%player,%mPosA,%len,%dir,true); //true if ledge found, args. - start position, raycast length, search direction's vector, 'get Position' = true
		if(firstWord(%isLedge)) //if 'isLedge' true
		{
		//	talk("lim =" SPC %lim);
			%ready = true; //note: ready to return 'mPos'
			break; //stop loop here
		}
	}
	if(!%ready) //if ledge not found
		return ""; //stop here, and return nothing

	%hPosA = getWords(%isLedge,1,2) SPC getWord(%mPosA,2); //offset mPos forward to match Raycast hit position
	%nPosA = vectorAdd(%hPosA,vectorScale(%dir,-$Adventure::LedgeHangLen)); //offset hPos backward to match ideal climbing position
	return %nPosA; //return moved PosA
}

//Vertical Height Adjustment
function AdventureVerticalSnap(%player,%newHack,%posA,%lim)
{
	%nPos = vectorAdd(%newHack,"0 0 -1.325"); //Position from Hack Position
	%dist = mAbs(getWord(%newHack,2)-getWord(%posA,2)); //vertical distance between snap position and hack position
	%snapPos = (%dist < (0.2*%lim) && %dist > 0) ? %nPos : ""; //if dist. less than 0.2 * 'limit' (e.g. 0.2*10, where 0.2 = 1 plate high) - define snapPos as nPos, else - leave it undefined

//	if(%snapPos !$= "")
//		talk("Vertical Snap relative distance =" SPC getWord(%newHack,2)-getWord(%posA,2));

	return %snapPos; //return %snapPos
}


/////////////////////
//Multi-Reach Check//
/////////////////////

//Climb Ledge Search (search vertically for ledges in 1-plate intervals)
function AdventureClimbLedgeFind(%player,%posA,%vec,%svec,%down)
{
	%brick = 0.6; //height of a brick
	%off = $Pref::Adventure::ClimbMax*%brick; //relative max climbing reach in plate units
	%max = $Pref::Adventure::ClimbMax*3; //max climbing reach in number of plates

	%scl = vectorScale(%svec,0.3); //scale the x add for predicting next ledge (0.5)
	%scl = $Pref::Adventure::BorderClimb ? "0 0 0" : %scl; //if Pref: Border-Climb on - nulify sideways velocity, else - keep it
	for(%a=0;%a<%max;%a++) //loop from a = 0, to a = %max-1
	{
		%mov = %down ? -(0.1+%off-(%a*0.2)) : (-0.1+%off-(%a*0.2)); //move z offset up/down 0.2 units (height of a plate) if %found DNE
	//	talk("off =" SPC %off);
	//	talk("mov[" @ %a @ "] =" SPC %mov);
	//	%mov = %down ? -0.2-(%a*0.2) : 0.2+(%a*0.2); //move z offset up/down 0.2 units (height of a plate) if %found DNE
		%sPos = vectorAdd(%posA,"0 0" SPC %mov); //offset position
	//	%sPos = vectorAdd(%sPos,%scl); //left/right ledge search offset
		%rPos = AdventureMultiLedgeCheck(%player,%sPos,%vec,true); //if ledge exists, relative hack position to ledge
		if(%rPos !$= "" && vectorDist("0 0" SPC getWord(%rPos,2),"0 0" SPC getWord(%posA,2)) > 0.1) //if we found a ledge
		{
			// %rcLen = 3*(3*0.2); //safety raycast length (3 bricks)
			// %rcEnd = %down ? -(4*(3*0.2)) : %rcLen; //if down bool = true - use negative (4 bricks) length, else - use positive one
			// %rcPos = getWords(%rPos,0,1) SPC getWord(%rPos,2) + %rcEnd; //make raycast endpoint vector, with scaled vertical adjustments
			// %next = !AdventureBeamCheck(%player,%posA,%rcPos,$TypeMasks::PlayerObjectType | $TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,"","Cl Ledge Find"); //no obstructing object found (in the way of the designated ledge)
			// if(%next) //if next bool = true
			// {
			//	%fPos = vectorAdd(%rPos,"0 0" SPC -1.325); //predicted player position
				%rPosZ = getWord(%rPos,2); //z component of next ledge-adjusted hack position (relative PosA)
				%hPosZ = getWord(%posA,2); //z component of hack position (posA)
				%mOff = (%rPosZ-%hPosZ)/2; //calculate 'middle Offset' for box check
				%mOff = %down ? %mOff-(2*%brick) : %mOff+0.2; //invert 'middle Offset' if 'down' is true
				%mPos = getWords(%rPos,0,1) SPC %hPosZ + %mOff; //make positional vector directly between 'posA' and 'rPos'
				%height = (5*%brick)+0.2*(%max-%a);
			//	talk("Hack Pos [" @ %a @ "] =" SPC %posA);
			//	talk("Middle Pos [" @ %a @ "] =" SPC %mPos);
			//	talk("Height [" @ %a @ "] =" SPC %height);
			//	talk("Height =" SPC %height);
				if(!%down) //if down arg. = false (Climbing Up)
					%groundUp = isObject(AdventureBoxCheck(%player,false,%height,%mPos,"","Climb Ledge Find","0 0.8 0.5")); //check Above for obstructing objects
				else //if down arg. = true (Climbing Down)
					%groundDown = isObject(AdventureBoxCheck(%player,false,%height,%mPos,"","Climb Ledge Find","0 0.5 0.8")); //check Below for obstructing objects
			//	talk("Ground Down [" @ %a @ "] =" SPC %groundDown);
				if((!%down && !%groundUp) || (%down && !%groundDown)) //if (Climbing Up, and ground Up DNE), or (Climbing Down, and Ground Down DNE)
				{
					%ready = true; //set 'ready' bool to true
					break; //stop loop
				}
			// }
		}
	}

	//Blockage found?
	if(!%ready) //if no ledge found, or ledge is obstructed
		return ""; //stop and return nothing

	//Find Add with SUVAT
	%m = 23.4; //23.4; //gravity multiplier
	%rSub = vectorSub(%rPos,%posA);
	%sLen = vectorLen(%rSub);
	%sLen = %down ? -%sLen : %sLen;
	%sMov = mSqrt(2*%m*%sLen); //use a SUVAT equation to calculate new vel
	%sMov = mCeil(%sMov*10)*0.1; //round up to nearest tenth (0.x)
	%time = 100+mAbs(%sLen)*100; //scale time to match traversal distance
//	%add = vectorScale(%sNrm,%sMov); //correct velocity add
//	%final = vectorAdd(%add,"0 0" SPC %sLen*0.5); //final velocity add (was 0.2)
	
//	talk("Climb Ledge Find:   time =" SPC %time);
//	talk("Climb Ledge Find:   zA =" SPC %sMov);
//	%sVel = vectorScale(%scl,4.5); //left/right diagonal climbing offset vel  (was 4.5, originally 2.8)
//	%vel = getWords(%sVel,0,1) SPC %zA; //combined final velocity
	%vel = "0 0" SPC %sMov; //%zA; //combined final velocity

	return %vel SPC %time SPC %rPos; //return new velocity, time, and relative PosA (hack)
}


//////////////////
//Edge Detection//
//////////////////

//Corner/Edge Check
function AdventureEdgeDetect(%player,%posA,%offVec,%type,%name)
{
	if(%type $= "ledge")
	{
		%oPosA = vectorAdd(%posA,vectorScale(%offVec,0.2)); //Offset vector relative to %offVec
		%isLedge = AdventureLedgeSearch(%player,%oPosA); //Offset'd Ledge search
		%result = %isLedge;
	}

	if(%type $= "block")
	{
		%fvec = %Player.getForwardVector();
		%fOffA = vectorAdd(%PosA,vectorScale(%fvec,$Adventure::LedgeHangLen-0.3)); //forward offset
		%startA = vectorAdd("0 0" SPC $Adventure::PosA_offset,%fOffA); //upward offset + forward offset
		%isBlock = AdventureBeamCheck(%player,%startA,0.8,$TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType,%offVec,"Edge Detect" SPC %name); //Offset vector-directed Ledge search
		%result = %isBlock;
	}

//	centerPrint(%player.client,"<font:impact:30><just:center>\c3Adventure Edge Detection" @ "<br><color:CCCCCC>isLedgeL =" SPC %isLedgeL @ "<br>isLedgeR =" SPC %isLedgeR @ "<br><color:CCCCCC>isBlockL =" SPC %isBlockL @ "<br>isBlockR =" SPC %isBlockR,5);
	return %result; //return first word true if (offset ledge DNE) and last word true if (perpendicular wall exists) //return true if (offset ledge DNE, or perpendicular wall exists)
}


//////////////////////
//Arc Raycast Search//
//////////////////////

//Arc Raycast Search
function AdventureArcSearch(%player,%start,%vec,%len,%masks,%n,%range,%fcn)
{
	for(%a=0;%a<=%n;%a++) //loop from a = 0, to a == %n
	{
		%step = (%range*2/%n); //the vertical offset between each %mov check (e.g. if %range = 0.25, and %n = 10; %step = 0.05)
		%mov = "0 1" SPC %range-(%a*%step); //adjusted new vec (e.g. if a = 1, n = 10, and range = 0.25; "0 1" SPC 0.25 - (1*(0.25 * 2 / 10)) = "0 1 0.2"
		%nrm = vectorNormalize(%mov); //normalize %mov vec
		%nvec = vectorRotateVector(%nrm,%vec); //rotate %nrm vec to match look axis
		%beam = vectorScale(%nvec,%len); //vector's scaled length
		%end = vectorAdd(%start,%beam); //end point
		%ray = containerRayCast(%start,%end,%masks,%player); //fire raycast and return result as %ray
		if(isObject(firstWord(%ray))) //if raycast hit something
		{
			%hit = posFromRaycast(%ray); //use hit position
			%found = eval(strReplace(%fcn,"%ray",%ray)); //found = the return of defined function (replaced the nominal %ray with real one)
			if(%found) //if ledge found
				break; //stop loop
		}
		else //otherwise
			%hit = %end; //use raycast length end position
		RaycastDebugUpdate(%player,%start,%hit,"Arc Search \[Case " @ %a @ "\]"); //Show Raycast Debugger
	}
}


//////////////
//Beam Check//
//////////////

//Beam Check
function AdventureBeamCheck(%player,%start,%len,%masks,%vec,%name)
{
	if(%name $= "Cl Ledge Find") //if called by "ClimbLedgeFind" fcn
		%end = %len; //use %len (it's actually %rPos)
	else //otherwise
	{
		%beam = vectorScale(%vec,%len); //ray final vector
		%end = vectorAdd(%start,%beam); //ray endpoint
	}
	%ray = containerRayCast(%start,%end,%masks,%player);

	%col = firstWord(%ray); //col object
	%found = isObject(%col) && %col.type !$= "ClimbHelper"; //found object, and object not a climb tool
	if(%found) //if %found is true
		%hit = posFromRaycast(%ray); //use hit position
	else if(%name $= "Aim Check") //otherwise if name = "Aim Check"
		%hit = vectorAdd(%start,vectorScale(%vec,1000));
	else  //otherwise
		%hit = %end; //use raycast endpoint
	RaycastDebugUpdate(%player,%start,%hit,"Beam Check" SPC %name);

	if(%name $= "Aim Check") //if called by Aim Check fcn
		%found = %hit; //use hit position instead
	return %found; //return %found (true/false bool)
}


/////////////
//Box Check//
/////////////

//Box Search
function AdventureBoxCheck(%player,%low,%height,%start,%width,%name,%color)
{
	%height = %height $= "" ? 0.1 : %height; //if height undefined - use 0.1, else - use height arg.
	%start = %start $= "" ? %player.getPosition() : %start; //if start undefined - use player pos, else - use start arg.
	%start = %low ? vectorAdd(%start,"0 0" SPC -%height/2) : %start; //if low arg. true - offset the starting point, else - keep start
	%width = %width $= "" ? 1.25 : %width; //if width undefined - use 1.25, else - use width arg. (1.245)

	BoxCheckDebugUpdate(%player,%start,%width,%height,%name,%color);
	
	%masks = $TypeMasks::fxBrickObjectType | $Typemasks::TerrainObjectType | $Typemasks::StaticShapeObjectType;
	initContainerBoxSearch(%start, %width SPC %width SPC %height, %masks);
	while(%col = containerSearchNext())
	{
		%num++;
		if(isObject(%col) && %col.type !$= "ClimbHelper")
		{
		//	if(%name $= "Climb Ledge Find")
		//		talk("Box Check:  Col [" @ %num @ "] =" SPC %col);
			return %col; //return col. object
		}
	}
	
	return 0; //return nothing
}


///////////////////////
//Default Vector Fcns//
///////////////////////

package AdventureVectors
{
	//Manually override muzzle vector
	function Player::getMuzzleVector(%player,%slot,%override)
	{
		%hPos = %player.AdvMuzzleEnd; //Muzzle Vector Fix endpoint
		if(%player.getDatablock().canAdventureAim && %player.client.ThirdPersonActive && !%override && %hPos !$= "") //if playertype can Aim, and client is in Third Person view, and not overriding, and hPos is defined
		{
			%mPos = %player.getMuzzlePoint(%slot); //muzzle point
			%sub = vectorSub(%hPos,%mPos); //vector b/n hit point and muzzle point
			%sNrm = vectorNormalize(%sub); //normalized sub vector
			return %sNrm; //use normalized sub vector instead
		}
		return Parent::getMuzzleVector(%player,%slot); //return result from Parent fcn
	}
	//Player - get Eye Vector (Look Vector)
	function Player::getEyeVector(%player)
	{
		%ParVec = Parent::getEyeVector(%player); //resultant vector from Parent fcn
	//	talk("Player - Get Eye Vector:  Parent Vector (old) =" SPC %ParVec);
		if(isObject(%player) && %player.getDatablock().canAdventure) //if player exists, and player data. is an Adventurer
		{
			%factor = 0.95; //largest allowed vector z
			%pvNrm = vectorNormalize(getWords(%ParVec,0,1) SPC 0); //normalized, horizontal parent vector
			%pvScl = vectorScale(%pvNrm,-(1-%factor)); //scale the normalized horizontal parent vector to a combined length of 1 - 'factor' (e.g. if 'factor' = 0.9, scale value = 0.1)
			if(getWord(%ParVec,2) == 1) //if looking straight up
				%ParVec = getWords(%pvScl,0,1) SPC %factor;
			else if(getWord(%ParVec,2) == -1) //otherwise, if looking straight down
				%ParVec = getWords(%pvScl,0,1) SPC -%factor;
			%ParVec = vectorNormalize(%ParVec); //normalize the complete vector (including the 0.99)
		//	talk("Player - Get Eye Vector:  Parent Vector (new) =" SPC %ParVec);
		}
		return %ParVec; //return the vector
	}
	//Camera - get Forward Vector (Look Vector)
	function Camera::getForwardVector(%camera)
	{
		%ParVec = Parent::getForwardVector(%camera); //resultant vector from Parent fcn
	//	talk("Camera - Get Forward Vector:  Parent Vector (old) =" SPC %ParVec);
		if(isObject(%camera.player) && %camera.player.getDatablock().canAdventure) //if camera's player exists, and player data. is an Adventurer
		{
			%factor = 0.99; //largest allowed vector z
			%pvNrm = vectorNormalize(getWords(%ParVec,0,1) SPC 0); //normalized, horizontal parent vector
			%pvScl = vectorScale(%pvNrm,-(1-%factor)); //scale the normalized horizontal parent vector to a combined length of 1 - 'factor' (e.g. if 'factor' = 0.9, scale value = 0.1)
			if(getWord(%ParVec,2) == 1) //if looking straight up
				%ParVec = getWords(%pvScl,0,1) SPC %factor;
			else if(getWord(%ParVec,2) == -1) //otherwise, if looking straight down
				%ParVec = getWords(%pvScl,0,1) SPC -%factor;
			%ParVec = vectorNormalize(%ParVec); //normalize the complete vector (including the 0.99)
		//	talk("Camera - Get Forward Vector:  Parent Vector (new) =" SPC %ParVec);
		}
		return %ParVec; //return the vector
	}
};activatePackage(AdventureVectors);


////////////////
//Camera Fixes//
////////////////

package AdventureCamera
{
	//Camera - Hide Observer Message - on Enter Orbit
	function SimObject::onCameraEnterOrbit(%player,%camera)
	{
		if(%player.getDatablock().canAdventure) //if player data. is an Adventurer
			%player.observerCount++; //Unparent observer message, but keep correct observer count
		else //otherwise
			Parent::onCameraEnterOrbit(%player,%camera); //Parent default fcn
	}
	//Camera - Hide Observer Message - on Leave Orbit
	function SimObject::onCameraLeaveOrbit(%player,%camera)
	{
		//Camera Speed Fix
		if($camera::movementSpeed !$= 40) //if camera movement speed is not 40
			$camera::movementSpeed = 40; //set it to 40

		//Observer Fix
		if(%player.getDatablock().canAdventure) //if player data. is an Adventurer
			%player.observerCount--; //Unparent observer message, but keep correct observer count
		else //otherwise
			Parent::onCameraLeaveOrbit(%player,%camera); //Parent default fcn
	}
	//Player on First Spawn (F8 Camera fix)
	function Armor::onAdd(%data,%player)
	{
		Parent::onAdd(%data,%player); //Parent default fcn (before)
		schedule(1,0,AdventureSetCamLocation,%player,%player.client.camera); //manually fix F8 position //was 50
	}
	//Camera F7 Bug Fix (for Climbing)
	function serverCmdDropPlayerAtCamera(%client)
	{
		if(isObject(%player = %client.player) && isObject(%camera = %client.camera)) //if client's player and camera both exist
			AdventureStopHang(%player,false,false,true); //Stop hang args. - not instant, no force, camera
		Parent::serverCmdDropPlayerAtCamera(%client); //Parent default fcn (between)
		%camera.AdvLastTrans = %camera.getTransform(); //save old camera transform
		%client.isCamF8 = false; //set cam F8 bool to false
	}
	//Camera F8 Bug Fix (for Climbing)
	function serverCmdDropCameraAtPlayer(%client)
	{
		if(isObject(%player = %client.player) && %player.isAdventureClimbing) //if player exists, and player is Climbing (up / down)
			AdventureStopHang(%player,false,true,true); //Stop hang args. - not instant, use force, camera
		Parent::serverCmdDropCameraAtPlayer(%client); //Parent default fcn (between)
	//	echo("F8:  cam mode =" SPC %client.camera.mode SPC "  cam transform =" SPC %client.camera.getTransform());
		%client.isCamF8 = true; //set cam F8 bool to true
	}
};activatePackage(AdventureCamera);