//The following functions can be copy-pasted directly into another Add-On, as-is (no renaming / editing necessary).

///////////////
//Projectiles//
///////////////

//Hata's Spawn Projectile (or Explosion)
function HataDoProjectile(%player,%explode,%data,%pos,%initVel,%scale)
{
	%p = new Projectile() //spawn projectile
	{
		dataBlock = %data; //use input data
		initialPosition = %pos; //use input position
		initialVelocity = %initVel; //use input velocity
		sourceObject = %player; //use input source object
		scale = %scale SPC %scale SPC %scale; //use input scale
	};MissionCleanup.add(%p); //mission editor hook
	if(%explode) //if explode arg. = true
		%p.explode(); //explode the projectile immediately
}


///////////////
//Vector Math//
///////////////

//Hata's Cardinal-ize Directional Vector 
function HataDirCheck(%vec) 
{
	//Align (North, South, East, West - to match Brick Grid)
	%vec = vectorNormalize(%vec); //normalize the input vector first
	%vecX = getWord(%vec,0) SPC "0 0"; //make new vector from x component only
	%vecY = 0 SPC getWord(%vec,1) SPC 0; //make new vector from y component only

	%vecXLen = vectorLen(%vecX); //measure length of x vector
	%vecYLen = vectorLen(%vecY); //measure length of y vector

	%dir = vectorNormalize(%vecX); //default dir to normalized x vector
	%dir = %vecYLen > %vecXLen ? vectorNormalize(%vecY) : %dir; //if y vector's length is longer - use normalized y vector, otherwise - keep normalized x vector
	return %dir; //return the newly made cardinal directional vector
}

//Hata's Relative Vector
function findRelativeX(%vec)
{
	%hVec = getWords(%vec,0,1) SPC 0; //build vector from horizontal component of input vector
	%nhVec = vectorNormalize(%hVec); //normalized horizontal vector
	return vectorCross(%nhVec,"0 0 1"); //return rightward vector
}

//Boodals Vector Manipulation
function vectorRotateVector(%vec1,%vec2)
{
	return vectorRotateAxis(%vec1,vectorToAxis(%vec2));
}
function vectorToAxis(%vec)
{
	return eulerToAxis(vectorToEuler(%vec));
}
function vectorToEuler(%vec)
{
	%vec = vectorNormalize(%vec);
	%yaw = mRadToDeg(mATan(-getWord(%vec,0),getWord(%vec,1)));
	%pitch = mRadToDeg(mASin(getWord(%vec,2)));
	return %pitch SPC 0 SPC %yaw;
}
function vectorRotateAxis(%vec,%axis)
{
	return matrixMulVector("0 0 0 " @ %axis,%vec);
}

//Trader's Rotation code (Euler & Axis)
function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}
function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

//Space Guy's Color Matching
function getClosestPaintColor(%rgba)
{
	%prevdist = 100000;
	%colorMatch = 0;
	for(%i = 0;%i < 64;%i++)
	{
		%color = getColorIDTable(%i);
		if(vectorDist(%rgba,getWords(%color,0,2)) < %prevdist && getWord(%rgba,3) - getWord(%color,3) < 0.3 && getWord(%rgba,3) - getWord(%color,3) > -0.3)
		{
			%prevdist = vectorDist(%rgba,%color);
			%colormatch = %i;
		}
	}
	return %colormatch;
}