//Stun Vehicle Datablock
datablock WheeledVehicleData(PhysicsStunVehicle)
{
	//Basic Settings
	category = "Vehicles";
	shapeFile = "add-ons/Item_skis/deathvehicle.dts";
	emap = true;

	//Vehicle Settings
	numMountPoints = 1;
	maxDamage = 999999;
	destroyedLevel = 200;

	//Rigid Body
	mass = 200;
	massCenter = "0 0.1 0.7";	// Center of mass for rigid body
	massBox = "0.6 0.8 1";	// Size of box used for moment of inertia

	//Physics
	drag = 0.2;
	density = 1;
	integration = 4; 			// Physics integration: Tick Sec/Rate
	bodyFriction = 1;
	bodyRestitution = 0.0;

	//Collision
	minImpactSpeed = 0.1;		// Impacts over this invoke the script callback
	minRunOverSpeed = 1;  	// how fast you need to be going to run someone over (do damage)
	runOverDamageScale = 100; //how much damage running over does
	collisionTol = 0.4;			// Collision distance tolerance (was 0.25)

	isSled = true;				//if its a sled, the wing surfaces dont work unless its on the ground
};
//Vehicle on Driver Dismount
function PhysicsStunVehicle::onDriverLeave(%data,%vehicle,%player)
{
	schedule(1,0,PSDelete,%vehicle); //delete Stun vehicle after 1 ms
}
function PhysicsStunVehicle::onImpact(%this,%vehicle,%null,%vec,%speed)
{
	%player = %vehicle.getMountedObject(0); //vehicle's driver
	if(!isObject(%player)) //if player DNE or vehicle is not Physics Jump Vehicle
		return; //stop here
	
	//Awesome but Inefficient
	// %hack = %player.getHackPosition(); //mid torso (starting point of raycast)
	// %len = 2; //final length of raycast
	// %masks = $TypeMasks::PlayerObjectType | $TypeMasks::fxBrickObjectType | $TypeMasks::TerrainObjectType; //raycast targets
	// %vel = %player.getVelocity(); //player velocity (victim velocity)
	// %nrm = vectorNormalize(%vel); //normalize the vector to have a magnitude of 1, for scaling
	// %ray = ActionMeleeBeamCheck(%player,%hack,%len,%masks,%nrm); //fire raycast from eye point
	// if(%ray != 0) //if Raycast Function found an object
	// {
		// %col = firstWord(%ray);
		// %col = %col.getType() & $TypeMasks::PlayerObjectType ? %col : 0;
		
		// talk("col =" SPC %col);
		// talk("dot =" SPC vectorDot(%vec,%player.stunhitter.getforwardvector()));
		// if(%col != 0) //if object is sufficient
			// DoStunPhysics(%col,%player.StunHitter);
	// }
	
	%pos = %vehicle.getPosition();
	%player.unmount();
	%ratio = %speed*100;
	%player.damage("","",%ratio,$DamageType::Fall); //apply fall damage
	%sub = vectorSub(%player.StunHitter.getPosition(),%pos);
	%rot = RotationFromVector(%sub);
	%player.setTransform(%pos SPC %rot);
	%player.StunHitter = "";
	schedule(5,0,ActionMeleeCamRestore,%player.client,"by Physics Stun, on Impact"); //Attempt to stop the Fancy cam immediately
}
//Rotation from Vector
function RotationFromVector(%vec)
{
	%yaw = mATan(getWord(%vec,1),getWord(%vec,0));
	%fix = "0 0 -1" SPC %yaw;
	%eul = axisToEuler(%fix);
	%eulA = vectorAdd(%eul,"0 0 -90");
	%rot = eulerToAxis(%eulA);
	return %rot;
}

//Packaged Code (Updating Default fcn's)
package PhysicsStun
{
	//on Player Mount
	function Armor::onMount(%data,%player,%col,%slot)
	{
		if(%col.dataBlock $= "PhysicsStunVehicle") //if the object we're trying to mount to uses the "PhysicsStunVehicle" datablock
		{
			%player.setControlObject(%col); //put player in control of vehicle
			%player.setTransform("0 0 0" SPC "0 0 0 0"); //reset player rotation and position (inside vehicle)
			%player.setScale("1 1 1"); //set player to default scale
		}
		else //otherwise
			Parent::onMount(%data,%player,%col,%slot); //Parent default fcn
	}
	//Projectile on Hit (via Damage)
	function ProjectileData::Damage(%data,%proj,%col,%fade,%pos,%normal)
	{
		Parent::Damage(%data,%proj,%col,%fade,%pos,%normal); //Parent default functions
		%col.PSProInitVel = %proj.initialVelocity; //Store projectile's initial velocity to damaged player
	}
};activatePackage(PhysicsStun);

//Implement Stun Physics
function DoStunPhysics(%player,%hitter,%imVel)
{
	if(fileName(%player.getDatablock().shapeFile) !$= "m.dts") //if not a Blockhead
		return; //stop here
	
	%player.StunHitter = %hitter;
	
	//Visual
	%player.setActionThread(root); //movement thread
	%player.playthread(3,root); //Default Stun animation thread

	%player.playthread(2,root); //clear all threads just in case
	%player.playthread(1,root); //..
	%player.playthread(0,root); //..

	//General
	%data = PhysicsStunVehicle; //Vehicle Datablock
	%vel = %player.getVelocity(); //Player's current velocity (called 1ms after Stun)
	%tra = %player.getTransform(); //Player's transform (pos & rot)
	%rot = getWords(%tra,3,6); //Get player rotation from player transform
	%player.canDismount = false; //disable dismounting

	%pos = %player.getPosition();
	%offset = vectorAdd(%pos,"0 0" SPC 0.1); //scale ground offset (prevent Stun vehicle from spawning stuck in the ground)

	//Spawn Physics Stun Vehicle
	%vehicle = new WheeledVehicle() 
	{
		dataBlock = %data;
		player = %player;
		position = %offset;
	};MissionCleanup.add(%vehicle);
	%vehicle.schedule(1,mountObject,%player,0);
	%player.PSVehicle = %vehicle;

	//Vehicle Velocity Calculations
	%xyV = getWords(%vel,0,1) SPC 0; //horizontal components of vel
	%len = vectorLen(%xyV); //speed (length of horizontal vel)
	%fvec = %player.getForwardVector(); //use forward vector for rotation
	%fdot = vectorDot(%vel,%fvec); //relative forward velocity (dot product)

	%vec = %xyV; //set rotation velocity to match velocity
	if(%len < 2) //if horizontal speed less than 6
	{
		if(%len >= 1) //if horizontal speed greater than or equal to 1 (and less than 6)
			%nvec = vectorScale(vectorNormalize(%xyV),2); //scale it to 2
		else //otherwise (if horizontal speed less than 1)
			%nvec = %player.PSProInitVel !$= "" ? vectorNormalize(%player.PSProInitVel) : %fvec; //if Damaging Projectile's Initial Velocity exists - use normalized vector from it (fall away from shot), else - use player's forward vector (fall forward)
		%vec = vectorScale(%nvec,6); //multiply vector by 6
		%vel = %vec; //add this velocity
	}

	StunVehicleRotate(%player,%vehicle,%vec,%rot,%imVel); //Apply rotational velocity
	%vehicle.setVelocity(%vel); //apply global velocity
}

//Pitch Physics Stun Vehicle
function StunVehicleRotate(%player,%vehicle,%vel,%rot,%imVel)
{
	%pitch = 0.1; //down scale the rotational velocity
	%fvec = %player.getForwardVector(); //player's forward vector
	%imVel = %imVel $= "" ? %vel : %imVel; //if impact velocity undefined, use regular velocity
	%rvec = findRelativeX(%fvec); //player's rightward vector

	%rvY = vectorDot(%imVel,%fvec); //relative y component of velocity
	%rvX = vectorDot(%imVel,%rvec); //relative x component of velocity
	%ang = -%rvY*%pitch SPC %rvX*%pitch SPC 0; //complete angular vel (relative)
	%final = vectorRotateVector(%ang,%fvec); //align the relative angular velocity vector with the player's view (make it global)

	%pos = %vehicle.getPosition(); //Stun vehicle position
	%vehicle.setTransform(%pos SPC %rot); //move and rotate vehicle to match player's transform
	%vehicle.setAngularVelocity(%final); //set rotational velocity
//	talk("Physics Stun Rotate   impulse velocity =" SPC %imVel SPC "  pitch =" SPC %pitch);
//	echo("Physics Stun Rotate   impulse velocity =" SPC %imVel SPC "  pitch =" SPC %pitch);
}

//Stun Vehicle Delete
function PSDelete(%vehicle)
{
	if(isObject(%vehicle)) //if vehicle exists
		%vehicle.delete(); //delete it
}


//Hata's Relative Vector
function findRelativeX(%vec)
{
	%hVec = getWords(%vec,0,1) SPC 0; //build vector from horizontal component of input vector
	%nhVec = vectorNormalize(%hVec); //normalized horizontal vector
	return vectorCross(%nhVec,"0 0 1"); //return rightward vector
}

//Boodals Vector Manipulation
function vectorRotateVector(%vec1,%vec2)
{
	return vectorRotateAxis(%vec1,vectorToAxis(%vec2));
}
function vectorToAxis(%vec)
{
	return eulerToAxis(vectorToEuler(%vec));
}
function vectorToEuler(%vec)
{
	%vec = vectorNormalize(%vec);
	%yaw   = mRadToDeg(mATan(-getWord(%vec,0),getWord(%vec,1)));
	%pitch = mRadToDeg(mASin(getWord(%vec,2)));
	return %pitch SPC 0 SPC %yaw;
}
function vectorRotateAxis(%vec,%axis)
{
	return matrixMulVector("0 0 0 " @ %axis,%vec);
}

//Trader's Rotation code (Euler & Axis)
function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}