datablock ParticleData(PhysicsDeathParticle)
{
	textureName          = "./empty";
	dragCoefficient      = 0.1;
	gravityCoefficient   = -0.1;
	inheritedVelFactor   = 0.1;
	constantAcceleration = 0.0;
	lifetimeMS           = 1;
	lifetimeVarianceMS   = 0;
	useInvAlpha = false;

	colors[0]     = "1.0 1.0 1.0 1.0";
	colors[1]     = "1.0 1.0 1.0 1.0";
	colors[2]     = "1.0 1.0 1.0 1.0";

	sizes[0]      = 0.01;
	sizes[1]      = 0.02;
	sizes[2]      = 0.03;

	times[0] = 0.0;
	times[1] = 0.1;
	times[2] = 0.2;
};

datablock ParticleEmitterData(PhysicsDeathEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 5;
   thetaMax         = 20;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = PhysicsDeathParticle;
};

datablock WheeledVehicleData(PhysicsDeathVehicle)
{
	//tagged fields
	doSimpleDismount = false;		//just unmount the player, dont look for a free space
	maxDismountDist = 5;

   numMountPoints = 1;

   category = "Vehicles";
   shapeFile = "add-ons/Item_skis/deathvehicle.dts";
   emap = true;

   maxDamage = 1000.00;
   destroyedLevel = 10.00;

   uiName = "";
   rideable = false;

   maxSteeringAngle = 1.800;  // Maximum steering angle, should match animation
   tireEmitter = PhysicsDeathEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = false;         // Roll the camera with the vehicle?
   cameraMaxDist = 7.5;         // Far distance from vehicle
   cameraOffset = 4.4;        // Vertical offset from camera mount point
   cameraLag = 25.0;           // Velocity lag of camera
   cameraDecay = 1.75;  //0.75;      // Decay per sec. rate of velocity lag
   cameraTilt = 0.3201; //tilt adjustment for camera: ~20 degrees down

   numWheels = 0;
   
   // Rigid Body
   mass = 300;
   density = 5.0;
   massCenter = "0 -1 1";    // Center of mass for rigid body
   massBox = "1 1 1";         // Size of box used for moment of inertia,
                              // if zero it defaults to object bounding box
   drag = 0.2; //was 0.8                // Drag coefficient
   bodyFriction = 1.0; //was 0.21
   bodyRestitution = 0.2; //was 0.2
   minImpactSpeed = 0;        // Impacts over this invoke the script callback
   softImpactSpeed = 3;       // Play SoftImpact Sound
   hardImpactSpeed = 10;      // Play HardImpact Sound
   integration = 4;           // Physics integration: TickSec/Rate
   collisionTol = 0.25;        // Collision distance tolerance
   contactTol = 0.1;          // Contact velocity tolerance
   justcollided = 0;

isSled = true;	//if its a sled, the wing surfaces dont work unless its on the ground
	
   // Engine
   engineTorque = 0;       // Engine power
   engineBrake = 0;         // Braking when throttle is 0
   brakeTorque = 80000;        // When brakes are applied
   maxWheelSpeed = 30;        // Engine scale by current speed / max speed

	forwardThrust		= 00; //5000
	reverseThrust		= 00;
	lift				= 00;
	maxForwardVel		= 00;
	maxReverseVel		= 00;
	horizontalSurfaceForce	= 50; //50
	verticalSurfaceForce	= 50;
	rollForce		= 0; //was 9000
	yawForce		= 0; //was 9000
	pitchForce		= 0; //was 36000
	rotationalDrag	= 5; //1
	stallSpeed		= 0;

	minRunOverSpeed    = 4;   //how fast you need to be going to run someone over (do damage)
	runOverDamageScale = 0;   //when you run over someone, speed * runoverdamagescale = damage amt
	runOverPushScale   = 1.2; //how hard a person you're running over gets pushed
};

function PhysicsDeathVehicle::onDriverLeave(%this,%obj,%player)
{
	%obj.schedule(10,delete);
}


package PhysicsDeath
{
	function Armor::onMount(%this,%obj,%col,%slot)
	{
		%data = %obj.getDatablock();
		if(%col.dataBlock !$= "PhysicsDeathVehicle")
			return parent::onMount(%this,%obj,%col,%slot);
		else
		{
			if(%slot == 0)
				%obj.setControlObject(%col);
			else
				%obj.setControlObject(%obj);
			%obj.setTransform(%col.getPosition() SPC "0 0 0");
		}
	}
	function Armor::onDisabled(%data,%obj,%enabled)
	{
		parent::onDisabled(%data,%obj,%enabled);
		%obj.schedule(200,deathAnim);
		%obj.schedule(60000,Poof);
		%obj.activatePhysics();
		%obj.noAnim();
		%obj.isDead=1;
	}
	function Armor::onRemove(%data,%obj)
	{
		parent::onRemove(%data,%obj);
		if(isObject(%obj.pd))
			%obj.pd.delete();
	}
	function Player::noAnim(%obj)
	{
		%obj.setActionThread(root);
		%obj.playthread(3,root);
		%obj.playthread(2,root);
		%obj.playthread(1,root);
		%obj.playthread(0,root);
	}
	function Player::deathAnim(%obj)
	{
		%obj.schedule(50,setThreadDir,1,false);
		%obj.playthread(3,headside);
		%obj.playthread(2,crouch);
		%obj.playthread(1,sit);
	}
	function Player::Poof(%obj)
	{
		%obj.isDead=0;
	}
	function Player::activatePhysics(%obj)
	{
		//make a new ski vehicle and mount the player on it
		%client = %obj.client;
		%pos = %obj.getPosition();
		%trans = %obj.getTransform();
		%words = getWords(%trans,3,6);
		%add = vectorAdd(%pos,"0 0 1") SPC %words;
		%obj.canDismount = false;

		%pd = new WheeledVehicle() 
		{
			dataBlock = PhysicsDeathVehicle;
			client = %client;
		};MissionCleanup.add(%pd);
		%pd.schedule(50,setTransform,%add);
		%pd.schedule(50,mountObject,%obj,0);
		%obj.schedule(5,setPhysicsDeathVelocity);
		%obj.schedule(60000,dissapear);
		%obj.schedule(60000,unmount);
		%obj.pd = %pd;
	}
	function Player::setPhysicsDeathVelocity(%obj)
	{
		%velA = %obj.getVelocity();
		%velB = %obj.pd.getVelocity();
		%velC = vectorAdd(%velA,%velB);
		%obj.pd.setVelocity(vectorScale(%velC,1));
	//	%obj.pd.setVelocity(vectorScale(%velC,2));
	}
	function Player::Dissapear(%obj)
	{
		%obj.setNodeColor("ALL","");
	}
};activatePackage(PhysicsDeath);

$CorpseTimeoutValue = 60000;